package org.hiroyukki.bxp.ciji;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.hiroyukki.bxp.cipu.CiPai;
import org.hiroyukki.bxp.cipu.CiPu;
import org.hiroyukki.bxp.config.ConfigReader;

/**
 * ʼĹ࣬´ʣȡǰĴʶҪ
 * 
 * @author hiroyukki
 * @since 1.0
 */
public class CiJiManager {

	/**
	 * get CiJi by ciJiName
	 * 
	 * @param ciJiName
	 *            String
	 * @return CiJi
	 */
	public static CiJi getCiJiFromName(String ciJiName) {
		File f = new File("ciji/" + ciJiName + ".ciji");
		try {
			FileInputStream fis = new FileInputStream(f);
			ObjectInputStream ois = new ObjectInputStream(fis);
			try {
				CiJi cj = (CiJi) ois.readObject();
				return cj;
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				return null;
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * create new CiJi by ciJiName and author
	 * 
	 * @param ciJiName
	 *            String
	 * @param author
	 *            String
	 */
	public static void createNewCiJi(String ciJiName, String author) {
		CiJi cj = new CiJi();
		cj.setAuthor(author);
		cj.setCiJiName(ciJiName);
		ObjectOutputStream oos = null;
		try {
			FileOutputStream fos = new FileOutputStream("ciji/" + ciJiName
					+ ".ciji");
			oos = new ObjectOutputStream(fos);
			oos.writeObject(cj);
			ConfigReader.getInstance().getCiJiSet().add(ciJiName);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				oos.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * ʼдһ
	 * 
	 * @param cj
	 *            CiJi
	 */
	public static void writeAgain(CiJi cj) {
		try {
			FileOutputStream fos = new FileOutputStream("ciji/"
					+ cj.getCiJiName() + ".ciji");
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(cj);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * add a new Ci into CiJi
	 * 
	 * @param cj
	 *            CiJi
	 * @param c
	 *            Ci
	 */
	public static void addNewCi(CiJi cj, Ci c) {
		cj.getCiList().add(c);
		writeAgain(cj);
	}

	/**
	 * delete a Ci from ciji
	 * 
	 * @param cj
	 *            CiJi
	 * @param c
	 *            Ci
	 */
	public static void removeCiFromCiJi(CiJi cj, Ci c) {
		cj.getCiList().remove(c);
		writeAgain(cj);
	}

	/**
	 * ͨһ״ʵõõĴ
	 * 
	 * @param c
	 *            Ci
	 * @return CiPu
	 */
	public static CiPu getCiPuFromCi(Ci c) {
		String ciPaiName = c.getCiPaiName();
		String type = c.getType();
		CiPai ciPai = new CiPai(ciPaiName);
		return ciPai.get(type);
	}
}
