package org.hiroyukki.bxp.io.poem;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.hiroyukki.bxp.config.ConfigReader;
import org.hiroyukki.bxp.poem.Poem;
import org.hiroyukki.bxp.poem.ShiJi;

/**
 * This class is used for serialize
 * 
 * @author hiroyukki
 * 
 */
public class ShiJiManager {
	/**
	 * get shi ji from shi ji name
	 * 
	 * @param name
	 * @return ShiJi
	 */
	public static ShiJi getShiJiByName(String name) {
		File f = new File("ciji/" + name + ".shiji");
		try {
			FileInputStream fis = new FileInputStream(f);
			ObjectInputStream ois = new ObjectInputStream(fis);
			try {
				ShiJi sj = (ShiJi) ois.readObject();
				return sj;
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				return null;
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Create a new shi ji and write out into file.
	 * 
	 * @param name
	 *            shi ji name
	 * @param author
	 *            author of shi ji
	 */
	public static void createShiJi(String name, String author) {
		ShiJi sj = new ShiJi();
		sj.setAuthor(author);
		sj.setName(name);
		ObjectOutputStream oos = null;
		try {
			FileOutputStream fos = new FileOutputStream("ciji/" + name
					+ ".shiji");
			oos = new ObjectOutputStream(fos);
			oos.writeObject(sj);
			ConfigReader.getInstance().getShiJiSet().add(name);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				oos.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Write out a shi ji into file
	 * 
	 * @param sj
	 */
	public static void write(ShiJi sj) {
		try {
			FileOutputStream fos = new FileOutputStream("ciji/" + sj.getName()
					+ ".shiji");
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(sj);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Insert a poem into shi ji and write it to file again
	 * 
	 * @param sj
	 *            ShiJi
	 * @param p
	 *            Poe
	 */
	public static void insertPoem(ShiJi sj, Poem p) {
		sj.add(p);
		write(sj);
	}

	/**
	 * remove a poem from shi ji and write it again into file
	 * 
	 * @param sj
	 *            ShiJi
	 * @param p
	 *            Poem
	 */
	public static void removePoem(ShiJi sj, Poem p) {
		sj.getPoemList().remove(p);
		write(sj);
	}
}
