package org.hiroyukki.bxp.socket;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;

import org.hiroyukki.bxp.socket.message.ChatMessage;
import org.hiroyukki.bxp.ui.main.MainMenuBar;
import org.hiroyukki.bxp.ui.socket.ChatPanel;

/**
 * a handle of socket, Server will start a handle for each connection
 * 
 * @author hiroyukki
 * @since 1.1
 */
public class SocketHandle implements Runnable {

	// private ServerSocket ss;
	private Socket s;
	private OutputStream out;
	private InputStream in;
	private ObjectOutputStream outObj;
	private ObjectInputStream inObj;

	/**
	 * init a socket handler
	 * @param serverSocket 
	 * @param socket
	 */
	public SocketHandle(ServerSocket serverSocket, Socket socket) {
		// this.ss = serverSocket;
		this.s = socket;
		try {
			out = s.getOutputStream();
			in = s.getInputStream();
			inObj = new ObjectInputStream(in);
			outObj = new ObjectOutputStream(out);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void run() {
		ChatPanel cp = new ChatPanel(this);
		MainMenuBar.getInstance().handlePanel(cp);
	}

	/**
	 * handle the event that recieve a chat message
	 * 
	 * @return ChatMessage get message from client
	 */
	public ChatMessage recieveMessage() {
		try {
			ChatMessage cm = (ChatMessage) inObj.readObject();
			return cm;
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}

	public void sendMessage(ChatMessage message) {
		try {
			outObj.writeObject(message);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
