package org.hiroyukki.bxp.ui.ciji;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.hiroyukki.bxp.ciji.Ci;
import org.hiroyukki.bxp.ciji.CiJi;
import org.hiroyukki.bxp.ciji.CiJiManager;
import org.hiroyukki.bxp.config.ConfigReader;
import org.hiroyukki.bxp.ui.main.MainPanel;
import org.hiroyukki.tools.ui.ComponentManager;

/**
 * This class provide some operations for view ci ji.<br>
 * This class is TINGLETON.
 * 
 * @author hiroyukki
 * @since 1.0
 */
public class CiJiPanel extends JPanel implements MainPanel, ActionListener {

	private static final long serialVersionUID = 1L;

	/**
	 * instance of CiJiPanel
	 */
	public static CiJiPanel instance;

	/**
	 * static method, get the single instance of CiJiPanel
	 * 
	 * @return CiJiPanel
	 */
	public static CiJiPanel getInstance() {
		if (instance == null) {
			instance = new CiJiPanel();
		}
		return instance;
	}

	private JSplitPane center = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
	private JPanel leftPanel = new JPanel();
	private JPanel rightPanel;
	private JPanel leftTopPanel = new JPanel();
	private JPanel leftBottomPanel = new JPanel();
	private JButton createNewCiJiButton = new JButton("½ʼ");
	private JButton deleteSelectedCiButton = new JButton("ɾ");
	private JButton madeCiJiBeTXTButton = new JButton("ı");
	private JButton madeCiBeTXTButton = new JButton("ı");
	private JPanel leftButtonPanel = new JPanel();
	private JPanel buttonPanel = new JPanel();
	private JList ciList = new JList();
	private DefaultListModel ciListModel;
	private JList ciJiList = new JList();
	private DefaultListModel ciJiListModel;

	/**
	 * Point out current ci will be use
	 */
	private Ci currentCi;
	/**
	 * Point out current ci ji
	 */
	private CiJi currentCiJi;
	/**
	 * store all ci in current ci ji into this map, in order take out ci quickly
	 * when show target ci.
	 */
	private Map<Integer, Ci> currentCiMap = new HashMap<Integer, Ci>();

	public CiJiPanel() {
		ciJiListModel = new DefaultListModel();
		ciListModel = new DefaultListModel();
		int i = 0;
		for (String ciJiName : ConfigReader.getInstance().getCiJiSet()) {
			ciJiListModel.add(i, ciJiName);
			i++;
		}
		ciJiList.setModel(ciJiListModel);
		ciList.setModel(ciListModel);
		// ѡһʼʱбҪı
		ciJiList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				String ciJiName = (String) ciJiList.getSelectedValue();
				currentCiJi = CiJiManager.getCiJiFromName(ciJiName);
				ciListModel.clear();
				currentCiMap.clear();
				try {
					// ǰʼҪı
					int i = 0;
					for (Ci c : currentCiJi.getCiList()) {
						ciListModel.add(i, c.getCiPaiName());
						currentCiMap.put(i, c);
						i++;
					}
					currentCi = null;
				} catch (Exception e1) {
					ComponentManager.msgBox("ĴʼʽĿ¼ɾ");
				}
			}
		});

		ciList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				int i = ciList.getSelectedIndex();
				currentCi = currentCiMap.get(i);
				remove(rightPanel);
				try {
					if (currentCi != null) {
						rightPanel = new AdmireCiPanel(currentCi);
					}
				} catch (Exception ex) {
					ComponentManager.msgBox("ʽʧܣӴɾϹĴʣ");
				}
				center.setRightComponent(rightPanel);
				validate();
			}
		});

		createNewCiJiButton.addActionListener(this);
		deleteSelectedCiButton.addActionListener(this);
		madeCiBeTXTButton.addActionListener(this);
		madeCiJiBeTXTButton.addActionListener(this);

		buttonPanel.setLayout(new GridLayout(1, 2, 5, 5));
		buttonPanel.add(createNewCiJiButton);
		buttonPanel.add(madeCiJiBeTXTButton);

		rightPanel = new JPanel();
		leftTopPanel.setBorder(new TitledBorder("ѡʼ"));
		leftTopPanel.setLayout(new BorderLayout());
		leftTopPanel.add(new JLabel("ѡһʼ"), BorderLayout.NORTH);
		leftTopPanel.add(new JScrollPane(ciJiList), BorderLayout.CENTER);
		leftTopPanel.add(buttonPanel, BorderLayout.SOUTH);
		leftBottomPanel.setBorder(new TitledBorder("б"));
		leftBottomPanel.setLayout(new BorderLayout());
		leftBottomPanel.add(new JScrollPane(ciList), BorderLayout.CENTER);
		leftButtonPanel.setLayout(new GridLayout(1, 2, 5, 5));
		leftButtonPanel.add(deleteSelectedCiButton);
		leftButtonPanel.add(madeCiBeTXTButton);
		leftBottomPanel.add(leftButtonPanel, BorderLayout.SOUTH);
		leftPanel.setLayout(new BorderLayout());
		leftPanel.add(leftTopPanel, BorderLayout.NORTH);
		leftPanel.add(leftBottomPanel, BorderLayout.CENTER);

		setLayout(new BorderLayout());
		center.setDividerLocation(186);
		center.setDividerSize(15);
		center.setResizeWeight(0);
		center.setOneTouchExpandable(true);
		center.setRightComponent(rightPanel);
		center.setLeftComponent(leftPanel);
		add(center, BorderLayout.CENTER);
	}

	/**
	 * ½ʼ
	 */
	public void handleCreateCiJi() {
		CreateJiFrame.getInstance().clear();
		CreateJiFrame.getInstance().setVisible(true);
		CreateJiFrame.getInstance().setCreateType(CreateJiFrame.CJ);
	}

	/**
	 * insert a ciji name into list after create ciji
	 * 
	 * @param str
	 *            name of ci ji
	 */
	public void addCiJiNameIntoList(String str) {
		ciJiListModel.addElement(str);
	}

	/**
	 * delete selected ci from ciji
	 */
	public void deleteSelectedCi() {
		if (currentCi == null) {
			ComponentManager.msgBox("ѡһ״ʣ");
		} else {
			if (JOptionPane.showConfirmDialog(null, "ȷҪɾ") == JOptionPane.YES_OPTION) {
				CiJiManager.removeCiFromCiJi(currentCiJi, currentCi);
				int i = ciList.getSelectedIndex();
				ciListModel.remove(i);
				currentCiMap.remove(i);
			}
		}
	}

	/**
	 * made ciji be TXT and show it.
	 */
	public void madeCiJiBeTXT() {
		if (currentCiJi == null) {
			ComponentManager.msgBox("ѡһʼ");
		} else {
			try {
				ShowContentWindow.getInstance().showContent(
						currentCiJi.toString());
				if (ConfigReader.getInstance().isMadeTXTFileWhenShowCiJi()) {
					try {
						BufferedWriter bw = new BufferedWriter(new FileWriter(
								"ciji/" + currentCiJi.getCiJiName() + ".txt"));
						bw.write(currentCiJi.toString());
						bw.flush();
						bw.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			} catch (Exception e) {
				ComponentManager.msgBox("ĴʼʽĿ¼ɾ");
			}
		}
	}

	/**
	 * made ci be TXT and show it
	 */
	public void madeCiBeTXT() {
		if (currentCi == null) {
			ComponentManager.msgBox("ѡһ״ʣ");
		} else {
			if (ConfigReader.getInstance().isMadeTXTFileWhenShowCiJi()) {
				try {
					BufferedWriter bw = new BufferedWriter(new FileWriter(
							"ciji/" + currentCi.getCiPaiName() + ".txt"));
					bw.write(currentCi.toString());
					bw.flush();
					bw.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			ShowContentWindow.getInstance().showContent(currentCi.toString());
		}
	}

	@Override
	public boolean canClose() {
		return true;
	}

	@Override
	public String getPanelName() {
		return "ʼ";
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == createNewCiJiButton) {
			handleCreateCiJi();
		} else if (e.getSource() == deleteSelectedCiButton) {
			deleteSelectedCi();
		} else if (e.getSource() == madeCiBeTXTButton) {
			madeCiBeTXT();
		} else if (e.getSource() == madeCiJiBeTXTButton) {
			madeCiJiBeTXT();
		}
	}
}
