package org.hiroyukki.bxp.ui.cipu;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.hiroyukki.bxp.cipu.CiPaiNameRedirector;
import org.hiroyukki.bxp.cipu.CiPuManager;
import org.hiroyukki.bxp.ui.main.MainPanel;
import org.hiroyukki.tools.ui.ComponentManager;
import org.hiroyukki.tools.ui.HTextArea;
import org.hiroyukki.tools.ui.HTextField;

/**
 * This class provide edit ci pai redirect functions for user, user can appoint
 * which ci pai name will redirect to another on, for example, "" will
 * redirect to "".<br>
 * This class is SINGLETON
 * 
 * @author hiroyukki
 * @version 2010.01.06
 * @since 2010
 */
public class CiPaiRedirectPanel extends JPanel implements ActionListener,
		MainPanel {

	private static final long serialVersionUID = 1L;

	static Properties pro = CiPaiNameRedirector.pro;
	public static CiPaiRedirectPanel instance;
	private static Integer I = 0;

	public static CiPaiRedirectPanel getInstance() {
		if (instance == null) {
			synchronized (I) {
				if (instance == null) {
					instance = new CiPaiRedirectPanel();
				}
			}
		}
		return instance;
	}

	private JSplitPane center = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
	private JPanel rightPanel = new JPanel();
	private JPanel leftPanel = new JPanel();
	JList ciPaiList = new JList();
	DefaultListModel cpListModel = new DefaultListModel();
	private String currentCiPai;
	private JPanel btnPanel = new JPanel();
	private JPanel listPanel = new JPanel();
	private JLabel infoLabel = new JLabel(" ");

	private HTextField sourceField = new HTextField(8);
	private HTextField targetField = new HTextField(8);
	private HTextArea ciPaiArea = new HTextArea();
	private JButton okBtn = new JButton("ȷ");
	private JButton deleteBtn = new JButton("ɾ");
	private JPanel topPanel = new JPanel();
	private JPanel areaPanel = new JPanel();
	private int numOfRedirect;

	private CiPaiRedirectPanel() {

		deleteBtn.addActionListener(this);
		okBtn.addActionListener(this);
		targetField.addActionListener(this);
		sourceField.addActionListener(this);
		setLayout(new BorderLayout());
		topPanel.setBorder(new TitledBorder("ҪĴƣ"));
		topPanel.add(new JLabel("ѣ"));
		topPanel.add(sourceField);
		topPanel.add(new JLabel("򵽣"));
		topPanel.add(targetField);
		topPanel.add(okBtn);
		areaPanel.setLayout(new BorderLayout());
		areaPanel.setBorder(new TitledBorder("ضĴƣ"));
		ciPaiArea.setEditable(false);
		ciPaiArea.setLineWrap(true);
		rightPanel.setLayout(new BorderLayout());
		areaPanel.add(new JScrollPane(ciPaiArea));
		rightPanel.add(areaPanel, BorderLayout.CENTER);
		rightPanel.add(topPanel, BorderLayout.NORTH);

		leftPanel.setLayout(new BorderLayout());
		btnPanel.add(deleteBtn);
		listPanel.setLayout(new BorderLayout());
		listPanel.setBorder(new TitledBorder("Ĵƣ"));
		listPanel.add(new JScrollPane(ciPaiList), BorderLayout.CENTER);
		listPanel.add(infoLabel, BorderLayout.SOUTH);
		leftPanel.add(listPanel, BorderLayout.CENTER);
		leftPanel.add(btnPanel, BorderLayout.SOUTH);

		add(center, BorderLayout.CENTER);
		center.setDividerLocation(186);
		center.setDividerSize(15);
		center.setResizeWeight(0);
		center.setOneTouchExpandable(true);
		center.setRightComponent(rightPanel);
		center.setLeftComponent(leftPanel);

		load();

		ciPaiList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				currentCiPai = (String) ciPaiList.getSelectedValue();
				if (currentCiPai != null) {
					infoLabel.setText("򵽣" + pro.get(currentCiPai));
				}
			}
		});
	}

	/**
	 * fill content into ci pai area from properties.
	 */
	@SuppressWarnings("unchecked")
	private void load() {
		Vector<String> v = new Vector(pro.keySet());
		ciPaiArea.setText("");
		cpListModel.clear();
		int i = 0;
		for (String str : v) {
			cpListModel.add(i, str);
			i++;
			ciPaiArea.append(str);
			ciPaiArea.append("=");
			ciPaiArea.append(pro.get(str).toString());
			ciPaiArea.append("\n");
		}
		ciPaiList.setModel(cpListModel);
	}

	/**
	 * delete ci pai name that selected
	 */
	private void deleteCiPai() {
		if (currentCiPai == null) {
			ComponentManager.msgBox("ѡһ");
		} else {
			pro.remove(currentCiPai);
			load();
			infoLabel.setText(" ");
		}
	}

	/**
	 * ǷֵУ<br>
	 * ˵򡰺ɡ<br>
	 * 򡰺ɡٱض򣬱ض·л·ڶಽضжջ
	 * 
	 * @param ciPaiName
	 * @return boolean ֵдڴ˴档
	 */
	public boolean ciPaiNameInValue(String ciPaiName) {
		return pro.containsValue(ciPaiName);
	}

	/**
	 * ǷڼС<br>
	 * ѴڣȻٽһζĶ
	 * 
	 * @param ciPaiName
	 *            ҪĴ
	 * @return boolean а˴ƣ档
	 */
	public boolean ciPaiNameInKey(String ciPaiName) {
		return pro.containsKey(ciPaiName);
	}

	/**
	 * ˷ضƵս˵̫ˡ<br>
	 * ˵Ϻáνϡνϡֶ򵽡ϡ<br>
	 * ӦֱʹϺá򵽡ϡȻڱ༭ʱһжϣ<br>
	 * ÿʱ˵ݹʽĴƶ
	 * 
	 * @param ciPaiName
	 *            ضеתƣ
	 * @return String Ĵ
	 */
	public String redirectToEnd(String ciPaiName) {
		// propertiesвļ˵ٶֱӷԭֵ
		if (!pro.containsKey(ciPaiName)) {
			return ciPaiName;
		}
		numOfRedirect++;
		String str = pro.getProperty(ciPaiName);
		if (numOfRedirect > 50) {
			ComponentManager.msgBox("ض࣬ܲ·\nҪĿơ");
			return null;
		}
		return redirectToEnd(str);
	}

	/**
	 * Ϊ·ѭڽضʱȽ˷Լض<br>
	 * ضֵжΪ·ؿա<br>
	 * ûв· 򷵻ض
	 * 
	 * @param target
	 *            ҪĴ
	 * @return String յõĶ
	 */
	private String handleRedirect(String target) {
		numOfRedirect = 0;
		String str = redirectToEnd(target);
		numOfRedirect = 0;
		return str;
	}

	/**
	 * ¼ӵһתԴѾתбеļУ򲻽ӡ<br>
	 * ԴƴתбеֵУҲӡ<br>
	 * ߽ʾ<br>
	 * ȻredirectToEnd(String ciPaiName)ս
	 */
	private void handleRedirect() {
		String source = sourceField.getText();
		String target = targetField.getText();
		if (source.equals("") || target.equals("")) {
			ComponentManager.msgBox("ԴƺĿƶΪա");
			return;
		}
		if (source.equals(target)) {
			ComponentManager.msgBox("Լж");
			return;

		}
		if (ciPaiNameInKey(source)) {
			ComponentManager.msgBox("Դѽй");
			return;
		}
		if (ciPaiNameInValue(source)) {
			ComponentManager.msgBox("Դĳ¼Ķ\nܽض");
			return;
		}
		String end = handleRedirect(target);
		if (end == null) {
			return;
		}
		if (!end.equals(target)) {
			ComponentManager.msgBox("" + target + "ѱ\n" + source
					+ "ض򵽣" + end);
			targetField.setText(end);
		}
		redirect(source, end);
	}

	/**
	 * жҪжĿǷڡ
	 * 
	 * @param source
	 *            String Դ
	 * @param target
	 *            String Ŀ
	 */
	private void redirect(String source, String target) {
		if (CiPuManager.hasCiPu(target)) {
			pro.put(source, target);
			load();
		} else {
			ComponentManager.msgBox("ĿƲڣȴҪĴƣ");
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == okBtn || e.getSource() == targetField
				|| e.getSource() == sourceField) {
			handleRedirect();
		} else if (e.getSource() == deleteBtn) {
			deleteCiPai();
		}
	}

	@Override
	public boolean canClose() {
		return true;
	}

	@Override
	public String getPanelName() {
		return "ƶ";
	}

	// public static void main(String[] args) {
	// String str = redirectToEnd("");
	// System.out.println(str);
	// }
}
