package org.hiroyukki.bxp.ui.config;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.hiroyukki.bxp.config.ConfigReader;
import org.hiroyukki.bxp.ui.main.MainFrame;
import org.hiroyukki.bxp.ui.main.MainPanel;
import org.hiroyukki.tools.ui.HTextField;

import com.sun.awt.AWTUtilities;

/**
 * 
 * 
 * @author hiroyukki
 * @since 1.0
 */
public class ConfigPanel extends JPanel implements MainPanel, ActionListener {

	private static final long serialVersionUID = 1L;

	public static ConfigPanel instance;

	public static ConfigPanel getInstance() {
		if (instance == null) {
			instance = new ConfigPanel();
		}
		return instance;
	}

	private JPanel topPanel = new JPanel();
	private JPanel buttonPanel = new JPanel();
	private JTabbedPane center = new JTabbedPane(JTabbedPane.RIGHT);
	private JPanel userPanel = new JPanel();
	private JPanel habbitPanel = new JPanel();
	private JButton saveButton = new JButton("");
	private JButton closeButton = new JButton("رմ");
	private HTextField userNameField = new HTextField(6);
	private JComboBox defaultCiJiNameBox;
	private JComboBox defaultShiJiBox;
	private JComboBox defaultYunTypeBox = new JComboBox(new String[] { "",
			"л" });
	private JCheckBox needWelcomePanelBox = new JCheckBox("");
	private JCheckBox exitWhenCloseBox = new JCheckBox("");
	private JCheckBox madeTXTWhenShowCiJiBox = new JCheckBox("");
	private JCheckBox needSocketServerBox = new JCheckBox("");
	private JComboBox defaultLAFTypeBox = new JComboBox(new String[] { "ĬϹ۸",
			"۸", "ϵͳ۸", "Motif۸" });
	private JSlider opacitySlider = new JSlider(JSlider.HORIZONTAL);

	/**
	 * JAVA LAF
	 */
	private String metalLAF = "javax.swing.plaf.metal.MetalLookAndFeel";
	/**
	 * LAF of system
	 */
	private String systemLAF = UIManager.getSystemLookAndFeelClassName();
	/**
	 * default LAF
	 */
	private String defaultLAF = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	private String motifLAF = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";

	public ConfigPanel() {
		initUserPanel();
		initHabbitPanel();
		saveButton.addActionListener(this);
		closeButton.addActionListener(this);
		opacitySlider.setValue(ConfigReader.getInstance().getOpacity());

		setLayout(new BorderLayout());
		topPanel.setLayout(new BorderLayout());
		topPanel.add(center, BorderLayout.CENTER);
		center.add(new JScrollPane(userPanel), "û");
		center.add(new JScrollPane(habbitPanel), "ƫ");
		buttonPanel.add(saveButton);
		buttonPanel.add(closeButton);
		topPanel.setBorder(new TitledBorder(""));
		buttonPanel.setBorder(new TitledBorder(""));
		add(topPanel, BorderLayout.CENTER);
		add(buttonPanel, BorderLayout.SOUTH);
	}

	/**
	 * ʼûƫ
	 */
	public void initHabbitPanel() {
		needWelcomePanelBox.setSelected(ConfigReader.getInstance()
				.isHasWelcomePanel());

		exitWhenCloseBox.setSelected(ConfigReader.getInstance()
				.isExitWhenClose());

		madeTXTWhenShowCiJiBox.setSelected(ConfigReader.getInstance()
				.isMadeTXTFileWhenShowCiJi());
		needSocketServerBox.setSelected(ConfigReader.getInstance()
				.isNeedSocketServer());

		if (ConfigReader.getInstance().getDefaultYunType().equals("")) {
			defaultYunTypeBox.setSelectedItem("");
		} else if (ConfigReader.getInstance().getDefaultYunType()
				.equals("л")) {
			defaultYunTypeBox.setSelectedItem("л");
		}

		if (ConfigReader.getInstance().getLookAndFeel().equals(defaultLAF)) {
			defaultLAFTypeBox.setSelectedItem("ĬϹ۸");
		} else if (ConfigReader.getInstance().getLookAndFeel()
				.equals(systemLAF)) {
			defaultLAFTypeBox.setSelectedItem("ϵͳ۸");
		} else if (ConfigReader.getInstance().getLookAndFeel().equals(metalLAF)) {
			defaultLAFTypeBox.setSelectedItem("۸");
		} else if (ConfigReader.getInstance().getLookAndFeel().equals(motifLAF)) {
			defaultLAFTypeBox.setSelectedItem("Motif۸");
		}

		JPanel needWelcomePanel = new JPanel();
		needWelcomePanel.add(new JLabel("ʱǷҪӭ壺"));
		needWelcomePanel.add(needWelcomePanelBox);

		JPanel defaultYunTypePanel = new JPanel();
		defaultYunTypePanel.add(new JLabel("ƫϣ"));
		defaultYunTypePanel.add(defaultYunTypeBox);

		JPanel exitWhenClosePanel = new JPanel();
		exitWhenClosePanel.add(new JLabel("ڹرʱǷ˳"));
		exitWhenClosePanel.add(exitWhenCloseBox);

		JPanel madeTXTWhenShowCiJiPanel = new JPanel();
		madeTXTWhenShowCiJiPanel.add(new JLabel("ɴıʱǷͬʱTXTļ"));
		madeTXTWhenShowCiJiPanel.add(madeTXTWhenShowCiJiBox);

		JPanel needSocketServerPanel = new JPanel();
		needSocketServerPanel.add(new JLabel("ǷҪ׽ַԶ̻õ˹ܣ"));
		needSocketServerPanel.add(needSocketServerBox);

		JPanel lafPanel = new JPanel();
		lafPanel.add(new JLabel("ۣ"));
		lafPanel.add(defaultLAFTypeBox);

		JPanel opacityPanel = new JPanel();
		opacityPanel.add(new JLabel("͸"));
		opacityPanel.add(opacitySlider);
		opacitySlider.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				int opacity = opacitySlider.getValue();
				AWTUtilities.setWindowOpacity(MainFrame.getInstance(),
						(float) (((float) opacity) / 100));
			}
		});

		habbitPanel.setLayout(new GridLayout(7, 1, 5, 5));
		habbitPanel.add(needWelcomePanel);
		habbitPanel.add(defaultYunTypePanel);
		habbitPanel.add(exitWhenClosePanel);
		habbitPanel.add(madeTXTWhenShowCiJiPanel);
		habbitPanel.add(needSocketServerPanel);
		habbitPanel.add(lafPanel);
		habbitPanel.add(opacityPanel);
	}

	/**
	 * ʼûϢ
	 */
	public void initUserPanel() {
		Vector<String> v = new Vector<String>(ConfigReader.getInstance()
				.getCiJiSet());
		Vector<String> v1 = new Vector<String>(ConfigReader.getInstance()
				.getShiJiSet());
		JPanel userNamePanel = new JPanel();
		JPanel ciJiNamePanel = new JPanel();
		defaultCiJiNameBox = new JComboBox(v);
		defaultShiJiBox = new JComboBox(v1);
		userPanel.setLayout(new BorderLayout());
		if (ConfigReader.getInstance().getUserName() != null) {
			userNameField.setText(ConfigReader.getInstance().getUserName());
		}
		if (ConfigReader.getInstance().getDefaultCiJiName() != null) {
			defaultCiJiNameBox.setSelectedItem(ConfigReader.getInstance()
					.getDefaultCiJiName());
		}
		if (ConfigReader.getInstance().getDefaultShiJi() != null) {
			defaultShiJiBox.setSelectedItem(ConfigReader.getInstance()
					.getDefaultShiJi());
		}

		JPanel defaultUserPanel = new JPanel();
		JPanel defaultCiJiPanel = new JPanel();
		JPanel defaultShiJiPanel = new JPanel();
		userNamePanel.setLayout(new GridLayout(6, 1, 5, 5));
		userNamePanel.add(defaultUserPanel);
		userNamePanel.add(defaultCiJiPanel);
		userNamePanel.add(defaultShiJiPanel);

		defaultUserPanel.add(new JLabel("Ĭû"));
		defaultUserPanel.add(userNameField);
		defaultCiJiPanel.add(new JLabel("Ĭϴʼ"));
		defaultCiJiPanel.add(defaultCiJiNameBox);
		defaultShiJiPanel.add(new JLabel("Ĭʫ"));
		defaultShiJiPanel.add(defaultShiJiBox);

		userPanel.add(userNamePanel, BorderLayout.NORTH);
		userPanel.add(ciJiNamePanel, BorderLayout.CENTER);
	}

	/**
	 * 
	 */
	public void saveConfig() {
		String userName = userNameField.getText();
		String defaultCiJiName = (String) defaultCiJiNameBox.getSelectedItem();
		String defaultShiJiName = (String) defaultShiJiBox.getSelectedItem();
		boolean isExitWhenClose = exitWhenCloseBox.isSelected();
		boolean isMadeTXTWhenShowCiJi = madeTXTWhenShowCiJiBox.isSelected();
		boolean needWelcome = needWelcomePanelBox.isSelected();
		boolean needSocketServer = needSocketServerBox.isSelected();
		int opacity = opacitySlider.getValue();
		String defaultYunType = (String) defaultYunTypeBox.getSelectedItem();
		String laf = null;
		if (defaultLAFTypeBox.getSelectedItem().equals("۸")) {
			laf = metalLAF;
		} else if (defaultLAFTypeBox.getSelectedItem().equals("ϵͳ۸")) {
			laf = systemLAF;
		} else if (defaultLAFTypeBox.getSelectedItem().equals("ĬϹ۸")) {
			laf = defaultLAF;
		} else if (defaultLAFTypeBox.getSelectedItem().equals("Motif۸")) {
			laf = motifLAF;
		}
		ConfigReader.getInstance().setOpacity(opacity);
		ConfigReader.getInstance().setUserName(userName);
		ConfigReader.getInstance().setDefaultCiJiName(defaultCiJiName);
		ConfigReader.getInstance().setDefaultShiJi(defaultShiJiName);
		ConfigReader.getInstance().setMadeTXTFileWhenShowCiJi(
				isMadeTXTWhenShowCiJi);
		ConfigReader.getInstance().setExitWhenClose(isExitWhenClose);
		ConfigReader.getInstance().setHasWelcomePanel(needWelcome);
		ConfigReader.getInstance().setDefaultYunType(defaultYunType);
		ConfigReader.getInstance().setLookAndFeel(laf);
		ConfigReader.getInstance().setNeedSocketServer(needSocketServer);
		try {
			UIManager.setLookAndFeel(laf);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}
		SwingUtilities.updateComponentTreeUI(MainFrame.getInstance());
		MainFrame.getInstance().center.remove(ConfigPanel.getInstance());
	}

	@Override
	public boolean canClose() {
		return true;
	}

	@Override
	public String getPanelName() {
		return "";
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == closeButton) {
			MainFrame.getInstance().center.remove(ConfigPanel.getInstance());
		} else if (e.getSource() == saveButton) {
			saveConfig();
		}
	}
}
