package org.hiroyukki.bxp.ui.load;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;

import org.hiroyukki.bxp.config.ConfigReader;
import org.hiroyukki.tools.ui.img.*;
import org.hiroyukki.tools.ui.ComponentManager;

/**
 * welcome window.When user be waiting for instantiation of application, show
 * this window to tell them application is running
 * 
 * @author hiroyukki
 * @since 1.1
 */
public class WelcomeWindow extends JWindow {

	private static final long serialVersionUID = 1L;

	public static WelcomeWindow instance;

	public static WelcomeWindow getInstance() {
		if (instance == null) {
			instance = new WelcomeWindow();
		}
		return instance;
	}

	private JProgressBar jpb = new JProgressBar();
	private ImageIcon bgImg = IconLoader.getImageIcon("lwq.jpg");
	private JLabel topLabel = new JLabel(bgImg);
	private List<LoadEvent> eventList = new ArrayList<LoadEvent>();
	private int allEvents;
	private int currentEvent;

	private WelcomeWindow() {
		setSize(400, 300);
		ComponentManager.moveToCenter(this);
		setLayout(new BorderLayout());
		add(topLabel, BorderLayout.CENTER);
		jpb.setStringPainted(true);
		add(jpb, BorderLayout.SOUTH);
		this.setVisible(true);
		addEvents();
		for (LoadEvent e : eventList) {
			handleTask(e);
		}
		this.setVisible(false);
		new Thread(new FadeOutThread()).start();
		// fadeOutMain();
		// MainFrame.getInstance().setVisible(true);
	}

	/**
	 * private void fadeOutMain() { int opacity =
	 * ConfigReader.getInstance().getOpacity();
	 * AWTUtilities.setWindowOpacity(MainFrame.getInstance(), 0);
	 * 
	 * for (int i = 0; i <= opacity; i++) {
	 * AWTUtilities.setWindowOpacity(MainFrame.getInstance(), (float) (((float)
	 * i) / 100)); try { Thread.sleep(1); } catch (InterruptedException e) {
	 * e.printStackTrace(); } } MainFrame.getInstance().setVisible(true); }
	 */

	/**
	 * Add all initializable event into list for ready handle.
	 */
	public void addEvents() {
		LoadEvent loadZhengYun = LoadCiLinZhengYun.getInstance();
		LoadEvent loadXinYun = LoadZhongHuaXinYun.getInstance();
		LoadEvent loadCiJi = LoadCiJi.getInstance();
		if (ConfigReader.getInstance().isNeedSocketServer() == true) {
			eventList.add(LoadSocket.getInstance());
		}
		LoadEvent loadUI = LoadUI.getInstance();
		eventList.add(loadZhengYun);
		eventList.add(loadXinYun);
		eventList.add(loadCiJi);
		eventList.add(loadUI);
		allEvents = eventList.size();
		currentEvent = 0;
	}

	/**
	 * handle every loadEvent from event list
	 * 
	 * @param evt
	 *            LoadEvent Load Event need be handle
	 */
	public void handleTask(LoadEvent evt) {
		jpb.setMaximum(allEvents);
		jpb.setString(evt.getMessage());
		evt.run();
		currentEvent++;
		jpb.setValue(currentEvent);
	}
}
