package org.hiroyukki.bxp.ui.main;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;

import org.hiroyukki.bxp.config.ConfigReader;
import org.hiroyukki.bxp.socket.Client;
import org.hiroyukki.bxp.socket.Server;
import org.hiroyukki.bxp.socket.SocketHandle;
import org.hiroyukki.bxp.ui.chm.HTMLCreaterPanel;
import org.hiroyukki.bxp.ui.ciji.CiJiPanel;
import org.hiroyukki.bxp.ui.cipu.CiEditPanel;
import org.hiroyukki.bxp.ui.cipu.CiPaiRedirectPanel;
import org.hiroyukki.bxp.ui.cipu.CiPuParsePanel;
import org.hiroyukki.bxp.ui.cipu.WillTianCiPanel;
import org.hiroyukki.bxp.ui.cipu.WriteCiPuPanel;
import org.hiroyukki.bxp.ui.config.ConfigPanel;
import org.hiroyukki.bxp.ui.poem.MakePoemPanel;
import org.hiroyukki.bxp.ui.poem.ShiJiPanel;
import org.hiroyukki.bxp.ui.yun.ShowYunPanel;
import org.hiroyukki.tools.ui.img.*;
import org.hiroyukki.tools.ui.ComponentManager;

/**
 * Ŀ¼
 * 
 * @author hiroyukki
 * @since 1.0
 */
public class MainMenuBar extends JMenuBar implements ActionListener {

	private static final long serialVersionUID = 1L;

	/**
	 * 
	 */
	public static MainMenuBar instance;

	/**
	 * õ
	 * 
	 * @return MainMenuBar
	 */
	public static MainMenuBar getInstance() {
		if (instance == null) {
			instance = new MainMenuBar();
		}
		return instance;
	}

	/**
	 * ϻ
	 */
	private String yunType;

	private JMenu fileMenu = new JMenu("ļ");
	private JMenu ciPuMenu = new JMenu("");
	private JMenu yunMenu = new JMenu("");
	private JMenu programMenu = new JMenu("");
	private JMenu otherMenu = new JMenu("");
	private JMenu writeMenu = new JMenu("");

	private JMenuItem versionItem = new JMenuItem("汾", IconLoader
			.getImageIcon("version.png"));
	private JMenuItem ciPuParseItem = new JMenuItem("׽", IconLoader
			.getImageIcon("parse.png"));
	private JMenuItem ciPaiDirectItem = new JMenuItem("ת", IconLoader
			.getImageIcon("jump.png"));
	private JMenuItem pingItem = new JMenuItem("Զ");
	private JMenuItem openItem = new JMenuItem("򿪴ʼ", IconLoader
			.getImageIcon("openCi.png"));
	private JMenuItem openShiJiItem = new JMenuItem("ʫ", IconLoader
			.getImageIcon("openShi.png"));
	private JMenuItem tianCiItem = new JMenuItem("", IconLoader
			.getImageIcon("writeCi.png"));
	private JMenuItem fuShiItem = new JMenuItem("ʫ", IconLoader
			.getImageIcon("writeShi.png"));
	private JMenuItem exitItem = new JMenuItem("˳", IconLoader
			.getImageIcon("exit.png"));
	private JMenuItem shutItem = new JMenuItem("رյǰ", IconLoader
			.getImageIcon("close.png"));
	private JMenuItem writeCiPuItem = new JMenuItem("д", IconLoader
			.getImageIcon("write.png"));
	private JMenuItem helpItem = new JMenuItem("", IconLoader
			.getImageIcon("help.jpg"));
	private JMenuItem authorItem = new JMenuItem("", IconLoader
			.getImageIcon("about.png"));
	private JMenuItem viewCiPuItem = new JMenuItem("鿴", IconLoader
			.getImageIcon("view.png"));
	private JMenuItem viewYunItem = new JMenuItem("鿴", IconLoader
			.getImageIcon("view.png"));
	private JMenuItem useYun = new JMenu("");
	private JMenuItem configItem = new JMenuItem("", IconLoader
			.getImageIcon("config.png"));
	private JMenuItem chmItem = new JMenuItem("HTML");
	private ButtonGroup bg = new ButtonGroup();
	private JRadioButton useZhengYunButton;
	private JRadioButton useXinYunButton;

	public MainMenuBar() {

		String yunType = ConfigReader.getInstance().getDefaultYunType();
		if (yunType.equals("")) {
			useZhengYunButton = new JRadioButton(" ", true);
			useXinYunButton = new JRadioButton("л ");
		} else {
			useZhengYunButton = new JRadioButton(" ");
			useXinYunButton = new JRadioButton("л ", true);
		}
		setYunType(yunType);

		bg.add(useZhengYunButton);
		bg.add(useXinYunButton);
		useYun.add(useZhengYunButton);
		useYun.add(useXinYunButton);

		ciPuParseItem.addActionListener(this);
		ciPaiDirectItem.addActionListener(this);
		openShiJiItem.addActionListener(this);
		openItem.addActionListener(this);
		tianCiItem.addActionListener(this);
		fuShiItem.addActionListener(this);
		exitItem.addActionListener(this);
		shutItem.addActionListener(this);
		helpItem.addActionListener(this);
		authorItem.addActionListener(this);
		viewYunItem.addActionListener(this);
		viewCiPuItem.addActionListener(this);
		configItem.addActionListener(this);
		writeCiPuItem.addActionListener(this);
		useZhengYunButton.addActionListener(this);
		useXinYunButton.addActionListener(this);
		pingItem.addActionListener(this);
		chmItem.addActionListener(this);
		versionItem.addActionListener(this);

		fileMenu.add(openItem);
		fileMenu.add(openShiJiItem);
		fileMenu.add(shutItem);
		fileMenu.addSeparator();
		fileMenu.add(exitItem);
		ciPuMenu.add(writeCiPuItem);
		ciPuMenu.add(viewCiPuItem);
		ciPuMenu.add(ciPaiDirectItem);
		// ciPuMenu.add(ciPuParseItem);
		writeMenu.add(tianCiItem);
		writeMenu.add(fuShiItem);
		yunMenu.add(useYun);
		yunMenu.add(viewYunItem);
		programMenu.add(configItem);
		programMenu.add(helpItem);
		programMenu.add(authorItem);
		programMenu.add(versionItem);
		otherMenu.add(pingItem);
		otherMenu.add(chmItem);

		add(fileMenu);
		add(writeMenu);
		add(ciPuMenu);
		add(yunMenu);
		// add(otherMenu);
		add(programMenu);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == viewYunItem) {
			handlePanel(ShowYunPanel.getInstance());
		} else if (e.getSource() == authorItem) {
			ComponentManager.msgBox("ϵhiroyukki@126.com", IconLoader
					.getImageIcon("author.png"));
		} else if (e.getSource() == exitItem) {
			MainFrame.getInstance().handleClose();
		} else if (e.getSource() == shutItem) {
			handleClosePanel();
		} else if (e.getSource() == tianCiItem) {
			handlePanel(WillTianCiPanel.getInstance());
		} else if (e.getSource() == viewCiPuItem) {
			handlePanel(WillTianCiPanel.getInstance());
		} else if (e.getSource() == openItem) {
			handlePanel(CiJiPanel.getInstance());
		} else if (e.getSource() == configItem) {
			handlePanel(ConfigPanel.getInstance());
		} else if (e.getSource() == writeCiPuItem) {
			handlePanel(new WriteCiPuPanel());
		} else if (e.getSource() == helpItem) {
			handlePanel(HelpPanel.getInstance());
		} else if (e.getSource() == useZhengYunButton) {
			setYunType("");
		} else if (e.getSource() == useXinYunButton) {
			setYunType("л");
		} else if (e.getSource() == pingItem) {
			handlePing();
		} else if (e.getSource() == chmItem) {
			handlePanel(HTMLCreaterPanel.getInstance());
		} else if (e.getSource() == fuShiItem) {
			handlePanel(new MakePoemPanel());
		} else if (e.getSource() == openShiJiItem) {
			handlePanel(ShiJiPanel.getInstance());
		} else if (e.getSource() == ciPaiDirectItem) {
			handlePanel(CiPaiRedirectPanel.getInstance());
		} else if (e.getSource() == ciPuParseItem) {
			handlePanel(CiPuParsePanel.getInstance());
		} else if (e.getSource() == versionItem) {
			ComponentManager.msgBox("\tCopyright   hiroyukki\n\t汾ţ1.3.2");
		}
	}

	/**
	 * ԶӴ
	 */
	public void handlePing() {
		String ipModel = "^(\\d{1,2}|1\\d\\d|2[0-4]\\d|25[0-5])(\\.(\\d{1,2}|1\\d\\d|2[0-4]\\d|25[0-5])){3}$";
		String ip = JOptionPane.showInputDialog("ҪӵIP", "");
		if (ip == null || ip.equals("") || !ip.matches(ipModel)) {
			ComponentManager.msgBox("ЧIPַ");
		} else {
			Client client = new Client(ip);
			new Thread(client).start();
			synchronized (client) {
				try {
					wait();
				} catch (InterruptedException e) {
					SocketHandle sh = new SocketHandle(Server.getIntance().ss,
							client.s);
					new Thread(sh).start();
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * رһ
	 */
	public boolean handleClosePanel() {
		JTabbedPane jtp = MainFrame.getInstance().center;
		Component cp = jtp.getSelectedComponent();
		if (cp instanceof MainPanel) {
			if (((MainPanel) cp).canClose()) {
				jtp.remove(cp);
				ComponentManager.shake(MainFrame.getInstance());
			} else {
				int i = JOptionPane.showConfirmDialog(null, "ĹûɣȷϹر");
				if (i == JOptionPane.YES_OPTION) {
					jtp.remove(cp);
					ComponentManager.shake(MainFrame.getInstance());
					if (cp instanceof WriteCiPuPanel) {
						((WriteCiPuPanel) cp).destroy();
					}
					if (cp instanceof CiEditPanel) {
						((CiEditPanel) cp).destroy();
					}
				} else {
					return false;
				}
			}
		}
		jtp.validate();
		return true;
	}

	/**
	 * 壬彫ʾѡϡ
	 * 
	 * @param mp
	 *            MainPanel
	 */
	public void handlePanel(MainPanel mp) {
		MainFrame.getInstance().center.add((Component) mp, mp.getPanelName());
		MainFrame.getInstance().center.setSelectedComponent((Component) mp);
	}

	/**
	 * 
	 * 
	 * @return String
	 */
	public String getYunType() {
		return yunType;
	}

	/**
	 * 
	 * 
	 * @param yunType
	 *            String
	 */
	public void setYunType(String yunType) {
		this.yunType = yunType;
	}
}
