package org.hiroyukki.bxp.ui.poem;

import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import org.hiroyukki.bxp.poem.PingShuiManager;
import org.hiroyukki.bxp.poem.PoemModel;
import org.hiroyukki.bxp.poem.ShiJu;
import org.hiroyukki.bxp.poem.ShiJuModel;
import org.hiroyukki.bxp.yun.CoreMethods;
import org.hiroyukki.bxp.yun.WordInfo;
import org.hiroyukki.bxp.yun.WordParserErrorException;
import org.hiroyukki.tools.ui.HTextArea;

/**
 * This class is for fu shi, you can fill content in this panel.
 * 
 * @author hiroyukki
 * @since 2010
 * @version 2010.01.06
 */
public class FuShiPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	public static String CLZY = "";

	private PoemModel poemModel;
	private List<ShiJuPanel> list;
	private HTextArea hta;
	/**
	 * 
	 */
	public WordInfo useYun;

	/**
	 * 
	 */
	public String wordsCanUse = "";

	public FuShiPanel(PoemModel poemModel, HTextArea hta) {
		this.hta = hta;
		this.poemModel = poemModel;
		setBorder(new TitledBorder(poemModel.name));
		list = new ArrayList<ShiJuPanel>();
		init();
	}

	/**
	 * set yun model and words that can use
	 * 
	 * @param wi
	 */
	public void setPingShui(WordInfo wi) {
		this.useYun = wi;
		wordsCanUse = PingShuiManager.getSamePingShui(wi);
		hta.setText("֣\n" + PingShuiManager.getSamePingShui(wi));
	}

	/**
	 * initialize components for a poem
	 */
	private void init() {
		for (ShiJuModel sj : poemModel.shiJuList) {
			list.add(new ShiJuPanel(sj, this));
		}
		setLayout(new GridLayout(list.size(), 1, 5, 5));
		for (ShiJuPanel sjp : list) {
			add(sjp);
		}
	}

	/**
	 * if all sentense passed, the whole poem cam pass.
	 * 
	 * @return boolean
	 */
	public boolean canPass() {
		boolean b = true;
		for (ShiJuPanel sjp : list) {
			b = b && sjp.pass;
			if (!b) {
				return b;
			}
		}
		return b;
	}

	/**
	 * clear all HTextFields.
	 */
	public void clear() {
		for (ShiJuPanel sjp : list) {
			sjp.clear();
		}
	}

	/**
	 * ʧճʧԣɣϵ״ҲǽɸԼ
	 */
	public String check() {
		StringBuilder sb = new StringBuilder();
		int i = 1;
		for (ShiJuPanel sjp : list) {
			sjp.checkAll();
			// һǵ䣬ǵһ䣬Ͼ֮ʧճ
			if (i % 2 == 1 && i > 1) {
				checkShiZhan(list.get(i - 2), sjp);
			}
			// һż䣬ϾʧԼ
			if (i % 2 == 0) {
				checkShiDui(list.get(i - 2), sjp);
			}
			sb.append("" + i + "䣺  ");
			sb.append(sjp.result);
			sb.append("\n");
			i++;
		}
		return sb.toString();
	}

	/**
	 * ʧճ<br>
	 * 찡ЩרҵӢͲã:(
	 * <p>
	 * ڽʫУһϾ䡪ô˵ʣ<br>
	 * ΪƽոĵһҲϾ䡪Ӧ˵ǵżһҪżճ<br>
	 * ǰֵƽҪͬ 磺<br>
	 * <br>
	 * һа졣<br>
	 * ǧѩ<br>
	 * Ųﴬ<br>
	 * ھ䣬ֻҪճͺã䣬ҪһаסֵƽҪֵ͡ƽͬ<br>
	 * ʵϵûôΪһ岻һԭڱеıһּۣƵҪϸ񣩣<br>
	 * ʵϾҪճֻеڶ֣硰һаסƽƣճƽƽƽҲǵڶճ<br>
	 * ֻжϵڶ֡<br>
	 * ټƽֵԵʣʧճжѾ΢ˣ<br>
	 * ϾǰڶֳɣӰ¾жϣʧճжֻǸο<br>
	 * ֻҪסһԭÿֶϸдǲʧճʧġ
	 * 
	 * @param sjp1
	 *            ż䣬ҪΪʧճж
	 * @param sjp2
	 *            䣬жϴ˾Ƿϸճ
	 * @return boolean ʧճ棬֮
	 */
	public boolean checkShiZhan(ShiJuPanel sjp1, ShiJuPanel sjp2) {
		/*
		 * һδдֱӷ
		 */
		if (!sjp1.isFull() || !sjp2.isFull()) {
			return true;
		}
		String word1 = sjp1.htf.getText().substring(1, 2);
		String word2 = sjp2.htf.getText().substring(1, 2);
		try {
			if (CoreMethods.isPingZeControversialWord(word1, CLZY)
					|| CoreMethods.isPingZeControversialWord(word2, CLZY)) {
				return true;
			}
			if (CoreMethods.getPingZe(word1, CLZY).equals(
					CoreMethods.getPingZe(word2, CLZY))) {
				return true;
			}
		} catch (WordParserErrorException e) {
			return true;
		}
		sjp2.result.append(" ʧճ");
		sjp2.message(sjp2.result.toString());
		return false;
	}

	/**
	 * ʧԡ<br>
	 * ʧָһгͶԾӦƽӦ෴<br>
	 * 硰ʹ֪ͣδ硱<br>
	 * 롰ƽƽƣƽƽƽ<br>
	 * Կ۵һ⣬ƽƵġ<br>
	 * ʧճжһҲֻǸοϸù࿼ʧԵ⡣<br>
	 * ȻڡСλܶԸá<br>
	 * ־ȵҪһЩжϡ<br>
	 * ʧճʧҪжϣʴڴнС<br>
	 * modify by hiroyukki at 2010.01.07:<br>
	 * һжϣһԴϾ䣬ƽոʫ<br>
	 * ڵһϾ䣬ԲӦüʧжϣΪ¾ҲϾ䣬Ӧöԡ
	 * 
	 * @param sjp1
	 *            
	 * @param sjp2
	 *            Ծ
	 * @return String ʧԵ֡
	 */
	public String checkShiDui(ShiJuPanel sjp1, ShiJuPanel sjp2) {
		// һûж
		if (!sjp1.isFull() || !sjp2.isFull()) {
			return "";
		}
		// һϾ䣬ζŲӦöԣ˼ⲻٽС
		if (sjp1.sj.isYunJu()) {
			return "";
		}
		StringBuilder sb = new StringBuilder();
		String str1 = sjp1.htf.getText();
		String str2 = sjp2.htf.getText();
		String word1;
		String word2;
		// 
		int length1 = str1.length();
		for (int i = 0; i < length1; i++) {
			// ǵһ֣
			if (i != 0 && i != 2) {
				word1 = str1.substring(i, i + 1);
				word2 = str2.substring(i, i + 1);
				try {
					// һƽ飬
					if (!(CoreMethods.isPingZeControversialWord(word1, CLZY) || CoreMethods
							.isPingZeControversialWord(word2, CLZY))) {
						// ƽͬ˵ԣʧֱ
						if (CoreMethods.getPingZe(word1, CLZY).equals(
								CoreMethods.getPingZe(word2, CLZY))) {
							sb.append(word2);
						}
					}
				} catch (WordParserErrorException e) {
					e.printStackTrace();
				}
			}
		}
		// ʧֱȲΪ㣬ʧıʧԾϢ
		if (sb.length() != 0) {
			sjp2.result.append("ʧԣ" + sb);
			sjp2.message(sjp2.result.toString());
		}
		return sb.toString();
	}

	/**
	 * õʫ
	 * 
	 * @return List<ShiJu>
	 */
	public List<ShiJu> getAllShiJu() {
		List<ShiJu> shiJuList = new ArrayList<ShiJu>();
		for (ShiJuPanel sjp : list) {
			shiJuList.add(sjp.getShiJu());
		}
		return shiJuList;
	}
}
