package org.hiroyukki.bxp.ui.poem;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.hiroyukki.bxp.cipu.CiPai;
import org.hiroyukki.bxp.cipu.CiPu;
import org.hiroyukki.bxp.config.ConfigReader;
import org.hiroyukki.bxp.io.poem.PoemModelReader;
import org.hiroyukki.bxp.io.poem.ShiJiManager;
import org.hiroyukki.bxp.poem.Poem;
import org.hiroyukki.bxp.poem.PoemModel;
import org.hiroyukki.bxp.poem.ShiJi;
import org.hiroyukki.bxp.ui.main.MainPanel;
import org.hiroyukki.tools.ui.ComponentManager;
import org.hiroyukki.tools.ui.HTextArea;
import org.hiroyukki.tools.ui.HTextField;

/**
 * ʫ壬ѡʫ͡
 * 
 * @author hiroyukki
 * @since 2010
 */
public class MakePoemPanel extends JPanel implements MainPanel, ActionListener {

	private static final long serialVersionUID = 1L;

	private boolean isSaved = false;
	private boolean isFirst = true;

	private JButton okButton = new JButton("ȷ");
	private JButton saveButton = new JButton("");
	private JButton clearButton = new JButton("д");
	private JButton checkButton = new JButton("");

	private JPanel poemInfoPanel = new JPanel();
	private JPanel remarkPanel = new JPanel();
	private JPanel leftTopPanel = new JPanel();
	private JPanel leftBottomPanel = new JPanel();
	private JList nameList = new JList();
	private DefaultListModel nameListModel;
	private JList typeList = new JList();
	private DefaultListModel typeListModel;
	private Map<String, CiPai> nameMap;
	/**
	 * ci pai that selected
	 */
	private CiPai currentCiPai;
	/**
	 * ci pu that selected
	 */
	private CiPu currentCiPu;

	private JPanel rightTopPanel = new JPanel();
	private JPanel rightBottomPanel = new JPanel();
	private JPanel leftPanel = new JPanel();
	private JPanel rightPanel = new JPanel();
	private JPanel areaPanel = new JPanel();
	private JSplitPane center = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
	private HTextArea hta;
	private HTextArea remarkArea;

	private HTextField authorField = new HTextField();
	private HTextField titleField = new HTextField();
	private JComboBox shiJiBox;

	private FuShiPanel fsp;
	private boolean isChecked = false;

	public MakePoemPanel() {

		setLayout(new BorderLayout());
		add(center, BorderLayout.CENTER);
		center.setLeftComponent(leftPanel);
		center.setRightComponent(rightPanel);
		center.setDividerSize(15);
		center.setDividerLocation(180);
		center.setOneTouchExpandable(true);
		center.setResizeWeight(0);
		leftPanel.setLayout(new GridLayout(2, 1, 5, 5));
		leftPanel.add(leftTopPanel);
		leftPanel.add(leftBottomPanel);
		leftTopPanel.setLayout(new BorderLayout());
		leftTopPanel.setBorder(new TitledBorder("ѡ"));
		leftTopPanel.add(nameList, BorderLayout.CENTER);
		leftBottomPanel.setLayout(new BorderLayout());
		leftBottomPanel.setBorder(new TitledBorder("ʽ"));
		leftBottomPanel.add(new JScrollPane(typeList), BorderLayout.CENTER);
		leftBottomPanel.add(okButton, BorderLayout.SOUTH);
		rightPanel.setLayout(new BorderLayout());
		rightTopPanel.setLayout(new BorderLayout());
		rightTopPanel.setBorder(new TitledBorder("ʫ"));
		rightBottomPanel.setLayout(new GridLayout(1, 7, 5, 5));
		rightBottomPanel.setBorder(new TitledBorder(""));
		rightBottomPanel.add(new JLabel());
		rightBottomPanel.add(clearButton);
		rightBottomPanel.add(new JLabel());
		rightBottomPanel.add(checkButton);
		rightBottomPanel.add(new JLabel());
		rightBottomPanel.add(saveButton);
		rightBottomPanel.add(new JLabel());
		areaPanel.setLayout(new BorderLayout());
		hta = new HTextArea(5, 0);
		hta.setEditable(false);
		hta.setLineWrap(true);
		JPanel htaPanel = new JPanel();
		htaPanel.setLayout(new BorderLayout());
		htaPanel.setBorder(new TitledBorder(""));
		htaPanel.add(new JScrollPane(hta), BorderLayout.CENTER);
		areaPanel.add(htaPanel, BorderLayout.CENTER);
		areaPanel.add(rightBottomPanel, BorderLayout.SOUTH);
		rightPanel.add(rightTopPanel, BorderLayout.CENTER);
		rightPanel.add(areaPanel, BorderLayout.SOUTH);

		remarkArea = new HTextArea();
		remarkArea.setLineWrap(true);
		remarkArea.append("ʾ\n");
		remarkArea.append("ﰴ»سģ塣\n");
		remarkArea.append("˴Ӧһʫıעᱣļ");
		remarkPanel.setBorder(new TitledBorder("ע"));
		remarkPanel.setLayout(new BorderLayout());
		remarkPanel.add(new JScrollPane(remarkArea), BorderLayout.CENTER);

		poemInfoPanel.setLayout(new GridLayout(6, 1, 3, 3));
		poemInfoPanel.setBorder(new TitledBorder("Ϣ"));
		poemInfoPanel.add(new JLabel("⣺"));
		titleField.setText("");
		poemInfoPanel.add(titleField);
		poemInfoPanel.add(new JLabel("ߣ"));
		if (ConfigReader.getInstance().getUserName() != null) {
			authorField.setText(ConfigReader.getInstance().getUserName());
		}

		poemInfoPanel.add(authorField);
		poemInfoPanel.add(new JLabel("ڣ"));

		nameListModel = new DefaultListModel();
		nameList.setModel(nameListModel);
		typeListModel = new DefaultListModel();
		typeList.setModel(typeListModel);

		okButton.setEnabled(false);
		saveButton.setEnabled(false);
		clearButton.setEnabled(false);
		checkButton.setEnabled(false);
		okButton.addActionListener(this);
		saveButton.addActionListener(this);
		clearButton.addActionListener(this);
		checkButton.addActionListener(this);

		Vector<String> v = new Vector<String>(ConfigReader.getInstance()
				.getShiJiSet());
		shiJiBox = new JComboBox(v);
		if (v.contains(ConfigReader.getInstance().getDefaultShiJi())) {
			shiJiBox.setSelectedItem(ConfigReader.getInstance()
					.getDefaultShiJi());
		}
		poemInfoPanel.add(shiJiBox);

		nameList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				typeListModel.clear();
				String name = (String) nameList.getSelectedValue();
				currentCiPai = nameMap.get(name);
				int i = 0;
				for (String type : currentCiPai.keySet()) {
					typeListModel.add(i, type);
					i++;
				}
			}
		});

		typeList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				String type = (String) typeList.getSelectedValue();
				currentCiPu = currentCiPai.get(type);
				okButton.setEnabled(true);
			}
		});

		init();
	}

	/**
	 * when this panel constract, load poem names from files
	 */
	private void init() {
		nameMap = PoemModelReader.getPoemModels();
		int i = 0;
		for (String str : nameMap.keySet()) {
			nameListModel.add(i, str);
			i++;
		}
	}

	/**
	 * when user confirm a type, load target poem model
	 */
	private void loadPoemModel() {
		if (isFirst) {
			if (currentCiPu != null) {
				PoemModel p = new PoemModel(currentCiPu);
				saveButton.setEnabled(true);
				clearButton.setEnabled(true);
				checkButton.setEnabled(true);
				fsp = new FuShiPanel(p, hta);
				rightTopPanel.add(new JScrollPane(fsp));
				okButton.setEnabled(false);
				isFirst = false;
				leftPanel.remove(leftTopPanel);
				leftPanel.remove(leftBottomPanel);
				leftPanel.add(new JScrollPane(poemInfoPanel));
				leftPanel.add(remarkPanel);
				validate();
			} else {
				ComponentManager.msgBox("ѡһ");
			}
		} else {
			ComponentManager.msgBox("ֻѡһΣҪд\nʫһµ塣");
		}
	}

	/**
	 * ڱǰӦȽһȫ飬鲻ͨ鱣棬ʱûȷǷִⱣ档
	 */
	private void beforeSave() {
		if (isChecked) {
			if (!fsp.canPass()) {
				if (JOptionPane.showConfirmDialog(null,
						"вϸʫ䣬鱣档\nִҪ") == JOptionPane.YES_OPTION) {
					save();
				}
			} else {
				save();
			}
		} else {
			if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(null,
					"δʧճʧԼ顣\nֱӱ")) {
				save();
			} else {
				check();
			}
		}
	}

	/**
	 * Save poem into shi ji file
	 */
	private void save() {
		if (isSaved) {
			if (JOptionPane.showConfirmDialog(null, "ѾҪ") == JOptionPane.YES_OPTION) {
				writeInFile();
			}
		} else {
			writeInFile();
			isSaved = true;
		}
	}

	/**
	 * Write out into files
	 */
	private void writeInFile() {
		String name = currentCiPai.getCiPaiName();
		String title = titleField.getText();
		String author = authorField.getText();
		String remark = remarkArea.getText();
		Poem p = new Poem();
		p.setList(fsp.getAllShiJu());
		p.setName(name);
		p.setType(currentCiPu.getType());
		p.setTitle(title);
		p.setAuthor(author);
		p.setRemark(remark);
		String shiJiName = (String) shiJiBox.getSelectedItem();
		ShiJi sj = ShiJiManager.getShiJiByName(shiJiName);
		ShiJiManager.insertPoem(sj, p);
		ComponentManager.msgBox("ɹ");
	}

	/**
	 * Clear all Field and reset message label
	 */
	private void clear() {
		fsp.clear();
	}

	/**
	 * о䣬Ѽдı
	 */
	private void check() {
		hta.setText(fsp.check());
		isChecked = true;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == okButton) {
			loadPoemModel();
		} else if (e.getSource() == saveButton) {
			beforeSave();
		} else if (e.getSource() == clearButton) {
			if (JOptionPane.showConfirmDialog(null, "ȷд") == JOptionPane.YES_OPTION)
				clear();
		} else if (e.getSource() == checkButton) {
			check();
		}
	}

	@Override
	public boolean canClose() {
		return isSaved;
	}

	@Override
	public String getPanelName() {
		return "ʫ";
	}
}