package org.hiroyukki.bxp.ui.poem;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.hiroyukki.bxp.config.ConfigReader;
import org.hiroyukki.bxp.io.poem.ShiJiManager;
import org.hiroyukki.bxp.poem.Poem;
import org.hiroyukki.bxp.poem.ShiJi;
import org.hiroyukki.bxp.ui.ciji.CreateJiFrame;
import org.hiroyukki.bxp.ui.ciji.ShowContentWindow;
import org.hiroyukki.bxp.ui.main.MainPanel;
import org.hiroyukki.tools.ui.ComponentManager;
import org.hiroyukki.tools.ui.HTextArea;

/**
 * Shi ji Panel, provide some operation of shi ji such as create a shi ji, read
 * out shi ji from file.<br>
 * THIS CLASS IS SINGLETON.
 * 
 * @author hiroyukki
 * @version 2010.01.06
 * @since 2010
 */
public class ShiJiPanel extends JPanel implements MainPanel, ActionListener {

	private static final long serialVersionUID = 1L;

	public static ShiJiPanel instance;
	private static final Integer I = 1;

	/**
	 * get singlton instance.
	 * 
	 * @return ShiJiPanel
	 */
	public static ShiJiPanel getInstance() {
		if (instance == null) {
			synchronized (I) {
				if (instance == null) {
					instance = new ShiJiPanel();
				}
			}
		}
		return instance;
	}

	private JSplitPane center = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);

	private JPanel leftPanel = new JPanel();
	private JPanel rightPanel = new JPanel();
	private JPanel leftTopPanel = new JPanel();
	private JPanel leftBottomPanel = new JPanel();
	private JPanel ltBtnPanel = new JPanel();
	private JPanel lbBtnPanel = new JPanel();
	JButton deleteShiBtn = new JButton("ɾʫ");
	JButton newShiJiBtn = new JButton("ʫ");
	JButton shiJiToTXTBtn = new JButton("ı");
	JButton shiToTXTBtn = new JButton("ı");
	private JList shiJiList = new JList();
	private DefaultListModel shiJiListModel;
	private JList shiList = new JList();
	private DefaultListModel shiListModel;

	private HTextArea poemArea = new HTextArea();
	private HTextArea remarkArea = new HTextArea();
	private JLabel infoLabel = new JLabel(" ");
	private JPanel poemPanel = new JPanel();

	private ShiJi currentShiJi;
	private Poem currentPoem;
	private Map<Integer, Poem> currentPoemMap;

	private ShiJiPanel() {
		ltBtnPanel.add(newShiJiBtn);
		ltBtnPanel.add(shiJiToTXTBtn);
		lbBtnPanel.add(deleteShiBtn);
		lbBtnPanel.add(shiToTXTBtn);
		leftTopPanel.setLayout(new BorderLayout());
		leftTopPanel.setBorder(new TitledBorder("ѡʫ"));
		leftTopPanel.add(new JScrollPane(shiJiList), BorderLayout.CENTER);
		leftTopPanel.add(ltBtnPanel, BorderLayout.SOUTH);
		leftBottomPanel.setLayout(new BorderLayout());
		leftBottomPanel.setBorder(new TitledBorder("ѡʫ"));
		leftBottomPanel.add(new JScrollPane(shiList), BorderLayout.CENTER);
		leftBottomPanel.add(lbBtnPanel, BorderLayout.SOUTH);
		leftPanel.setLayout(new GridLayout(2, 1, 5, 5));
		leftPanel.add(leftTopPanel);
		leftPanel.add(leftBottomPanel);

		rightPanel.setLayout(new BorderLayout());
		poemArea.setEditable(false);
		remarkArea.setEditable(false);
		infoLabel.setBorder(new TitledBorder("Ϣ"));
		rightPanel.add(infoLabel, BorderLayout.NORTH);
		poemArea.setBorder(new TitledBorder("ݣ"));
		remarkArea.setBorder(new TitledBorder("ע"));
		poemPanel.setLayout(new GridLayout(1, 2, 10, 10));
		poemPanel.add(new JScrollPane(poemArea));
		poemPanel.add(new JScrollPane(remarkArea));
		rightPanel.add(poemPanel, BorderLayout.CENTER);

		setLayout(new BorderLayout());
		add(center, BorderLayout.CENTER);
		center.setDividerLocation(186);
		center.setDividerSize(15);
		center.setResizeWeight(0);
		center.setOneTouchExpandable(true);
		center.setRightComponent(rightPanel);
		center.setLeftComponent(leftPanel);

		deleteShiBtn.addActionListener(this);
		newShiJiBtn.addActionListener(this);
		shiJiToTXTBtn.addActionListener(this);
		shiToTXTBtn.addActionListener(this);

		initList();
	}

	/**
	 * initialize shi ji list and shi list
	 */
	private void initList() {
		shiJiListModel = new DefaultListModel();
		shiListModel = new DefaultListModel();
		currentPoemMap = new HashMap<Integer, Poem>();
		int i = 0;
		for (String str : ConfigReader.getInstance().getShiJiSet()) {
			shiJiListModel.add(i, str);
		}
		shiJiList.setModel(shiJiListModel);
		shiList.setModel(shiListModel);

		shiJiList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				String shiJiName = (String) shiJiList.getSelectedValue();
				currentShiJi = ShiJiManager.getShiJiByName(shiJiName);
				shiListModel.clear();
				currentPoemMap.clear();
				int i = 0;
				for (Poem p : currentShiJi.getPoemList()) {
					shiListModel.add(i, p.getName()
							+ (p.getTitle().equals("") ? "" : ""
									+ p.getTitle()));
					currentPoemMap.put(i, p);
					i++;
				}
			}
		});

		shiList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				int i = shiList.getSelectedIndex();
				currentPoem = currentPoemMap.get(i);
				if (currentPoem != null) {
					handlePoem(currentPoem);
				}
			}
		});
	}

	/**
	 * handle a poem, show infomations of poem to ui.
	 * 
	 * @param p
	 *            Poem
	 */
	private void handlePoem(Poem p) {
		StringBuilder sb = new StringBuilder();
		if (!p.getTitle().equals("")) {
			sb.append("⣺" + p.getTitle());
		}
		sb.append("      ø");
		sb.append(p.getType());
		sb.append("      ڣ");
		sb.append(p.getDate());
		infoLabel.setText(sb.toString());
		poemArea.setText(p.toString());
		remarkArea.setText(p.getRemark());
	}

	/**
	 * make poem to txt and show out, if user selected need write out file in
	 * config, it also create a txt file in directory /ciji
	 */
	private void shiToTXT() {
		if (currentPoem == null) {
			ComponentManager.msgBox("ѡһʫ");
		} else {
			ShowContentWindow.getInstance().showContent(currentPoem.toString());
			if (ConfigReader.getInstance().isMadeTXTFileWhenShowCiJi()) {
				try {
					BufferedWriter bw = new BufferedWriter(new FileWriter(
							"ciji/" + currentPoem.getName() + ".txt"));
					bw.write(currentPoem.toString());
					bw.flush();
					bw.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * make poem to txt and show out,
	 * 
	 * @see shiToTXT()
	 */
	private void shiJiToTXT() {
		if (currentShiJi == null) {
			ComponentManager.msgBox("ѡһʫ");
		} else {
			ShowContentWindow.getInstance()
					.showContent(currentShiJi.toString());
			if (ConfigReader.getInstance().isMadeTXTFileWhenShowCiJi()) {
				try {
					BufferedWriter bw = new BufferedWriter(new FileWriter(
							"ciji/" + currentShiJi.getName() + ".txt"));
					bw.write(currentShiJi.toString());
					bw.flush();
					bw.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * create a new Shi ji and write in file, name of shi ji and author must be
	 * provide by user.
	 */
	private void createShiJi() {
		CreateJiFrame.getInstance().clear();
		CreateJiFrame.getInstance().setVisible(true);
		CreateJiFrame.getInstance().setCreateType(CreateJiFrame.SJ);
	}

	/**
	 * Add new shi ji name into list after create new si ji
	 * 
	 * @param str
	 *            name of new shi ji
	 */
	public void addShiJiNmeIntoList(String str) {
		shiJiListModel.addElement(str);
	}

	/**
	 * delete current poem in current shi ji
	 */
	private void deletePoem() {
		if (currentPoem == null) {
			ComponentManager.msgBox("ѡһʫ");
		} else {
			if (JOptionPane.showConfirmDialog(null, "ȷɾ") == JOptionPane.YES_OPTION) {
				ShiJiManager.removePoem(currentShiJi, currentPoem);
				int i = shiList.getSelectedIndex();
				shiListModel.remove(i);
			}
		}
	}

	@Override
	public boolean canClose() {
		return true;
	}

	@Override
	public String getPanelName() {
		return "ʫ";
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == deleteShiBtn) {
			deletePoem();
		} else if (e.getSource() == shiJiToTXTBtn) {
			shiJiToTXT();
		} else if (e.getSource() == shiToTXTBtn) {
			shiToTXT();
		} else if (e.getSource() == newShiJiBtn) {
			createShiJi();
		}
	}
}
