package org.hiroyukki.bxp.ui.poem;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

import org.hiroyukki.bxp.poem.ShiJu;
import org.hiroyukki.bxp.poem.ShiJuModel;
import org.hiroyukki.bxp.yun.CoreMethods;
import org.hiroyukki.bxp.yun.NotWordException;
import org.hiroyukki.bxp.yun.WordInfo;
import org.hiroyukki.bxp.yun.WordParserErrorException;
import org.hiroyukki.tools.ui.img.*;
import org.hiroyukki.tools.string.StringTranslator;
import org.hiroyukki.tools.ui.ComponentManager;
import org.hiroyukki.tools.ui.HTextField;

/**
 * This class is for a shi ju
 * 
 * @author hiroyukki
 * @since 2010
 * @version 2010.01.06
 */
public class ShiJuPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	public static final int ERROR = 4;
	public static final int WARN = 3;
	public static final int NORMAL = 1;
	public static final int PARSE = 2;
	public static String YUNTYPE = "";

	private int level = 0;
	/**
	 * ΪڸʫҪһǲϾжʧΪpublic
	 */
	public ShiJuModel sj;
	private FuShiPanel fsp;
	public HTextField htf = new HTextField();
	private JLabel infoLabel = new JLabel();

	public boolean pass = false;

	/**
	 * ֱ
	 */
	StringBuilder chuLvWords;

	/**
	 * ֱ
	 */
	StringBuilder zhengYiWords;

	/**
	 * ֱֲܽܽϳɺ͹ƽж
	 */
	private StringBuilder parseErrorWords;

	/**
	 * ƽֱʱ˱Ϊһƽ
	 */
	private StringBuilder pingWords;

	/**
	 * result of check all
	 */
	public StringBuilder result;

	public ShiJuPanel(ShiJuModel sj, FuShiPanel fsp) {
		this.fsp = fsp;
		this.sj = sj;
		setLayout(new GridLayout(1, 2, 10, 10));

		htf.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				String str = JOptionPane.showInputDialog("Ҫõ֣");
				if (!str.equals("")) {
					setPingShui(str);
				}
			}
		});

		htf.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent e) {
			}

			@Override
			public void focusLost(FocusEvent e) {
				checkAll();
			}
		});

		add(infoLabel);
		add(htf);
		message(sj.getContext() + " " + StringTranslator.isNull(sj.getInfo()));
	}

	/**
	 * ״̬ɺͳϡƽּ顣<br>
	 * ȶеݣǿգȲԣʾ<br>
	 * Ȼصȼӵ͵߽м飬ɣƽϡδܽ<br>
	 * ʧճʧеУԣдڸʫˡ
	 * 
	 * @see FuShiPanel
	 */
	public void checkAll() {
		String str = htf.getText();
		int length = str.length();
		// Ϊղ
		if ("".equals(str) || str == null) {
			return;
		}
		// Ȳϸʾһ
		if (length != sj.getLength()) {
			message("ȲϹ淶ʧܣ");
			level = NORMAL;
			return;
		}
		result = new StringBuilder();
		chuLvWords = new StringBuilder();
		zhengYiWords = new StringBuilder();
		parseErrorWords = new StringBuilder();
		pingWords = new StringBuilder();
		/* ȼǷг͹ƽ */
		checkLv(str);
		// 
		// ֻһƽ֣ƽ
		if (pingWords.length() <= 1) {
			result.append(" ƽ");
		}
		if (parseErrorWords.length() > 0) {
			result.append("ʧܣ" + parseErrorWords.toString());
			if (level < PARSE) {
				level = PARSE;
			}
		}
		if (zhengYiWords.length() > 0) {
			result.append("飺" + zhengYiWords.toString());
			if (level < WARN) {
				level = WARN;
			}
		}
		if (chuLvWords.length() > 0) {
			result.append("ɣ" + chuLvWords.toString());
			if (level < ERROR) {
				level = ERROR;
			}
		}

		/*
		 * һϾ䣬ҪӦϻ߳ϼ
		 */
		if (sj.isYunJu()) {
			/* ûȷϣƽˮϣгϼ */
			if (fsp.useYun == null) {
				String end = str.substring(length - 1, length);
				try {
					setPingShui(end);
				} catch (WordParserErrorException e) {
					parseErrorWords.append(end);
				}
			} else {
				String end = str.substring(length - 1, length);
				try {
					checkYun(end);
				} catch (WordParserErrorException e) {
					parseErrorWords.append(end);
				}
			}
		}

		// do a relevant attention after check
		if (result.length() == 0) {
			result.append("ɡ");
			level = 1;
		}
		if (level == 4) {
			pass = false;
		} else {
			pass = true;
		}
		warn(level);
		message(result.toString());
	}

	/**
	 * ɼ飬ÿһ飬ֳɣӵֱ
	 * 
	 * @param str
	 *            content in text field
	 */
	private void checkLv(String str) {
		int length = str.length();
		for (int i = 0; i < length; i++) {
			String word = str.substring(i, i + 1);
			try {
				checkWord(word, i);
			} catch (WordParserErrorException e) {
				parseErrorWords.append(word);
			}
		}
	}

	/**
	 * ˷ͬʱɺ͹ƽ<br>
	 * һ֣һӦλϵ֣Уɼ飬<br>
	 * ƽּ飬ƽ֣һҪƽ)ƽֱ<br>
	 * <br>
	 * ڶжǷƽ飬飬<br>
	 * ͬʱĬûõǴֵĶӦٰежƽ֣<br>
	 * һƽ֣ λӦƽּ֣ƽ<br>
	 * <br>
	 * жǷɣɣɱ<br>
	 * ɺ͹ƽͬǴѾˣҪǷƽ:)
	 * 
	 * @param word
	 *            ʫе֣Ǻ;ģУԡ
	 * @param i
	 *            еλ
	 * @throws WordParserErrorException
	 */
	private void checkWord(String word, int i) throws WordParserErrorException {
		// ϶ɣǷƽ
		if (sj.getContext().subSequence(i, i + 1).equals("")) {
			// жǷƽ飬ǣ塣ƽƽֱ
			if (!CoreMethods.isPingZeControversialWord(word, YUNTYPE)) {
				if ("ƽ".equals(CoreMethods.getPingZe(word, YUNTYPE))) {
					pingWords.append(word);
				}
			}
			return;
		}
		/**
		 * ִе˵ײУƽĻ϶飬 <br>
		 * ʱӦĬûõһȥģвҴֵ<br>
		 * ƽ Ϊ϶ˣ鿴漸䣩ƽֱ
		 */
		if (CoreMethods.isPingZeControversialWord(word, YUNTYPE)) {
			zhengYiWords.append(word);
			// ʫģеõӦλϵҪֵƽĬϴȡƽƽϱ
			if (sj.getContext().substring(i, i + 1).equals("ƽ")) {
				pingWords.append(word);
			}
			return;
		}
		/**
		 * ִе˵ײУַƽ飬ҪжֵƽƺУԡ<br>
		 * ۺУԽ ƽҪƽֱ
		 */
		String pingZe = CoreMethods.getWordInfo(word, YUNTYPE).getPingZe();
		if (pingZe.equals("ƽ")) {
			pingWords.append(word);
		}
		if (!pingZe.equals(sj.getContext().substring(i, i + 1))) {
			chuLvWords.append(word);
		}
	}

	/**
	 * ֱ֣жǷϡ<br>
	 * жֵ¼ֻҪһ¼Ҫͨ
	 * 
	 * @param word
	 *            last word is yun word, need be check yun
	 * @throws WordParserErrorException
	 */
	private void checkYun(String word) throws WordParserErrorException {
		if (CoreMethods.isControversialWord(word, YUNTYPE)) {
			WordInfo wi1 = CoreMethods.getWordInfo(word, YUNTYPE);
			WordInfo wi2 = CoreMethods.getWordInfo(word + "1", YUNTYPE);
			if (!(wi1.equals(fsp.useYun) || wi2.equals(fsp.useYun))) {
				result.append("ϣ");
				level = 4;
			}
		} else {
			WordInfo wi = CoreMethods.getWordInfo(word, YUNTYPE);
			if (!wi.equals(fsp.useYun)) {
				result.append("ϣ");
				level = 4;
			}
		}
	}

	/**
	 * 
	 * @param word
	 *            single word should set be yun model
	 * @throws WordParserErrorException
	 */
	private void setPingShui(String word) {
		if (result == null) {
			result = new StringBuilder();
		}
		try {
			/*
			 * һֱֲ֣óɹ
			 */
			if (!CoreMethods.isControversialWord(word, YUNTYPE)) {

				fsp.setPingShui(CoreMethods.getWordInfo(word, YUNTYPE));
				result.append("ģ壺" + word);
				return;
			} else {
				/*
				 * һƽ֣ѡɵһϣӦûѡʹĸϡ
				 */
				if (CoreMethods.isPingZeControversialWord(word, YUNTYPE)) {
					WordInfo wi1 = CoreMethods.getWordInfo(word, YUNTYPE);
					WordInfo wi2 = CoreMethods.getWordInfo(word + "1", YUNTYPE);
					if (wi1.getPingZe().equals(sj.getPingZe())) {
						fsp.setPingShui(wi1);
					} else {
						fsp.setPingShui(wi2);
					}
				} else {
					new SetYunFrame(word);
				}
			}
		} catch (WordParserErrorException e) {
			ComponentManager.msgBox("ʧܣܽ֡");
		} catch (NotWordException e) {
			ComponentManager.msgBox("ʧܣЧ֡");
		}
	}

	/**
	 * show message on label to call attention to user
	 * 
	 * @param str
	 *            content should be show out
	 */
	public void message(String str) {
		infoLabel.setText(str);
	}

	/**
	 * reset this component, clear text field and reset message label
	 */
	public void clear() {
		htf.setText("");
		message(sj.getContext() + " " + StringTranslator.isNull(sj.getInfo()));
	}

	/**
	 * жϵȼиıUI
	 */
	private void warn(int i) {
		switch (i) {
		case 3:
			htf.setBackground(new Color(250, 150, 0));
			break;
		case 4:
			htf.setBackground(Color.RED);
			break;
		case 1:
			htf.setBackground(Color.WHITE);
			break;
		case 2:
			htf.setBackground(Color.YELLOW);
			break;
		default:
			htf.setBackground(Color.WHITE);
			break;
		}
	}

	/**
	 * if length of model eqauls with length of field, return true
	 * 
	 * @return boolean
	 */
	public boolean isFull() {
		return (sj.getLength() == htf.getText().length());
	}

	/**
	 * ӵǰеõһʫ
	 * 
	 * @return ShiJu
	 */
	public ShiJu getShiJu() {
		ShiJu sj = new ShiJu();
		sj.setContent(htf.getText());
		return sj;
	}

	/**
	 * ҪϵʱһҲƽ֣ҪûֶϣʱҪˡ<br>
	 * ݴһ֣ϲƣֻҷƽ飩졣
	 * 
	 * @author hiroyukki
	 * @since 1.3
	 * @version 2010.01.06
	 */
	class SetYunFrame extends JFrame implements ActionListener {

		private static final long serialVersionUID = 1L;

		private String word;
		private WordInfo currentWordInfo;
		private WordInfo wi1;
		private WordInfo wi2;

		public SetYunFrame() {
			init();
		}

		public SetYunFrame(String word) {
			this.word = word;
			init();
		}

		private JButton okBtn = new JButton("ȷ");
		private JButton cancelBtn = new JButton("ȡ");
		private JPanel btnPanel = new JPanel();
		private JPanel topPanel = new JPanel();
		private JPanel mainPanel = new JPanel();
		private JRadioButton jrb1;
		private JRadioButton jrb2;
		private ButtonGroup bg = new ButtonGroup();

		private void init() {
			wi1 = CoreMethods.getWordInfo(word, YUNTYPE);
			wi2 = CoreMethods.getWordInfo(word + "1", YUNTYPE);
			jrb1 = new JRadioButton(wi1.getYunDetails());
			jrb2 = new JRadioButton(wi2.getYunDetails());
			bg.add(jrb1);
			bg.add(jrb2);
			topPanel.setLayout(new GridLayout(1, 2, 5, 5));
			topPanel.add(jrb1);
			topPanel.add(jrb2);
			btnPanel.add(okBtn);
			btnPanel.add(cancelBtn);
			mainPanel.setLayout(new BorderLayout());
			mainPanel.setBorder(new TitledBorder("ѡһ"));
			mainPanel.add(topPanel, BorderLayout.CENTER);
			mainPanel.add(btnPanel, BorderLayout.SOUTH);
			add(mainPanel);
			okBtn.addActionListener(this);
			cancelBtn.addActionListener(this);
			jrb1.addActionListener(this);
			jrb2.addActionListener(this);
			setIconImage(ImageLoader.getImage("logo.png"));
			setResizable(false);
			setTitle("ѡ");
			setSize(160, 160);
			ComponentManager.moveToCenter(this);
			setVisible(true);
		}

		/**
		 * ѵǰϢó
		 */
		private void setYun() {
			if (currentWordInfo == null) {
				ComponentManager.msgBox("ѡһϢ");
				return;
			}
			fsp.setPingShui(currentWordInfo);
			dispose();
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == okBtn) {
				setYun();
			} else if (e.getSource() == cancelBtn) {
				dispose();
			} else if (e.getSource() == jrb1) {
				currentWordInfo = wi1;
			} else if (e.getSource() == jrb2) {
				currentWordInfo = wi2;
			}
		}
	}
}
