package org.hiroyukki.bxp.ui.socket;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import org.hiroyukki.bxp.socket.SocketHandle;
import org.hiroyukki.bxp.socket.message.ChatMessage;
import org.hiroyukki.bxp.ui.main.MainPanel;
import org.hiroyukki.tools.ui.HTextArea;

/**
 * chat with friends, you can request get ciji or cipu from you friends
 * 
 * @author hiroyukki
 * @since 1.1
 */
public class ChatPanel extends JPanel implements MainPanel, ActionListener {

	private static final long serialVersionUID = 1L;
	private SocketHandle handle;
	private boolean canClose;

	public ChatPanel(SocketHandle handle) {
		this.handle = handle;
		initUI();
		canClose = true;
	}

	private JSplitPane center = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
	private HTextArea outputArea = new HTextArea();
	private HTextArea inputArea = new HTextArea(5, 0);
	private JPanel rightPanel = new JPanel();
	private JPanel leftPanel = new JPanel();
	private JPanel editMessagePanel = new JPanel();
	private JButton sendMessageButton = new JButton("");

	private void initUI() {
		setLayout(new BorderLayout());
		rightPanel.setLayout(new BorderLayout());
		outputArea.setLineWrap(true);
		outputArea.setEditable(false);
		inputArea.setLineWrap(true);
		sendMessageButton.addActionListener(this);
		editMessagePanel.setLayout(new BorderLayout());
		editMessagePanel.add(new JScrollPane(inputArea), BorderLayout.CENTER);
		rightPanel.add(new JScrollPane(outputArea), BorderLayout.CENTER);
		rightPanel.add(editMessagePanel, BorderLayout.SOUTH);
		center.setDividerLocation(150);
		center.setDividerSize(15);
		center.setResizeWeight(0);
		center.setOneTouchExpandable(true);
		center.setRightComponent(rightPanel);
		center.setLeftComponent(leftPanel);
		add(center, BorderLayout.CENTER);
	}

	public void sendMessage() {
		if (inputArea.getText() == "") {
			outputArea.append("ܷͿϢ\n\n");
		} else {
			ChatMessage cm = new ChatMessage();
			cm.setContent(inputArea.getText());
			cm.setTime(new Date());
			showMessage(cm);
			handle.sendMessage(cm);
		}
	}

	/**
	 * show chat message in TextArea
	 * 
	 * @param message
	 *            ChatMessage the message need to show
	 */
	public void showMessage(ChatMessage message) {
		outputArea.append(message.toString());
	}

	@Override
	public boolean canClose() {
		return canClose;
	}

	@Override
	public String getPanelName() {
		return "";
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == sendMessageButton) {
			sendMessage();
		} else {

		}
	}
}
