package org.hiroyukki.bxp.ui.yun;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.border.TitledBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeModel;

import org.dom4j.Document;
import org.dom4j.swing.DocumentTreeModel;
import org.hiroyukki.bxp.config.ConfigReader;
import org.hiroyukki.bxp.ui.main.MainPanel;
import org.hiroyukki.bxp.xml.DOM4jReader;
import org.hiroyukki.bxp.yun.CoreMethods;
import org.hiroyukki.bxp.yun.NotWordException;
import org.hiroyukki.bxp.yun.WordParserErrorException;
import org.hiroyukki.tools.ui.ComponentManager;
import org.hiroyukki.tools.ui.HTextArea;
import org.hiroyukki.tools.ui.HTextField;

/**
 * չʾϻлϵ
 * 
 * @author hiroyukki
 * @since 1.0
 */
public class ShowYunPanel extends JPanel implements MainPanel, ActionListener {

	private static final long serialVersionUID = 1L;

	/**
	 * 
	 */
	public static ShowYunPanel instance;

	/**
	 * õ
	 * 
	 * @return ShowYunPanel
	 */
	public static ShowYunPanel getInstance() {
		if (instance == null) {
			instance = new ShowYunPanel();
		}
		return instance;
	}

	private JSplitPane center = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
	private JPanel leftPanel = new JPanel();
	private JPanel rightPanel = new JPanel();
	private JPanel leftTopPanel = new JPanel();
	private JPanel leftBottomPanel = new JPanel();
	private JRadioButton zhengYunButton = new JRadioButton("");
	private JRadioButton xinYunButton = new JRadioButton("л");
	private ButtonGroup bg = new ButtonGroup();
	private JTree yunTree = new JTree();
	private TreeModel dtm;
	private Document doc;

	private HTextArea showWordsArea = new HTextArea();
	private HTextField getWordField = new HTextField(4);
	private ButtonGroup bg1 = new ButtonGroup();
	private JRadioButton yaYunButton = new JRadioButton("鿴ͬ", true);
	private JRadioButton xieYunButton = new JRadioButton("鿴Э");
	private JButton viewButton = new JButton("鿴");
	private JPanel rightTopPanel = new JPanel();
	private JPanel rightBottomPanel = new JPanel();

	public ShowYunPanel() {

		dtm = getTreeModel(getDefaultYunType());
		yunTree.setRootVisible(false);
		yunTree.setModel(dtm);

		zhengYunButton.addActionListener(this);
		xinYunButton.addActionListener(this);
		viewButton.addActionListener(this);
		getWordField.addActionListener(this);
		bg.add(zhengYunButton);
		bg.add(xinYunButton);
		leftTopPanel.setLayout(new GridLayout(2, 1, 5, 5));
		leftTopPanel.setBorder(new TitledBorder("ѡ"));
		leftTopPanel.add(zhengYunButton);
		leftTopPanel.add(xinYunButton);
		leftBottomPanel.setBorder(new TitledBorder(""));
		leftBottomPanel.setLayout(new BorderLayout());
		leftBottomPanel.add(new JScrollPane(yunTree), BorderLayout.CENTER);

		leftPanel.setLayout(new BorderLayout());
		leftPanel.add(leftTopPanel, BorderLayout.NORTH);
		leftPanel.add(leftBottomPanel, BorderLayout.CENTER);

		bg1.add(yaYunButton);
		bg1.add(xieYunButton);

		rightTopPanel.setBorder(new TitledBorder(""));
		rightBottomPanel.setBorder(new TitledBorder(""));
		JPanel jp = new JPanel();
		jp.setLayout(new GridLayout(1, 2, 5, 5));
		jp.add(getWordField);
		jp.add(viewButton);
		showWordsArea.setLineWrap(true);
		showWordsArea.setEditable(false);
		rightBottomPanel.setLayout(new BorderLayout());
		rightBottomPanel.add(new JScrollPane(showWordsArea));
		rightTopPanel.setLayout(new GridLayout(3, 1, 5, 5));
		rightTopPanel.add(jp);
		rightTopPanel.add(yaYunButton);
		rightTopPanel.add(xieYunButton);
		rightPanel.setLayout(new BorderLayout());
		rightPanel.add(rightTopPanel, BorderLayout.NORTH);
		rightPanel.add(rightBottomPanel, BorderLayout.CENTER);

		setLayout(new BorderLayout());
		add(center, BorderLayout.CENTER);
		center.setDividerLocation(400);
		center.setDividerSize(15);
		center.setOneTouchExpandable(true);
		center.setResizeWeight(1);
		center.setLeftComponent(leftPanel);
		center.setRightComponent(rightPanel);
	}

	/**
	 * ü
	 */
	{
		yunTree.addTreeSelectionListener(new TreeSelectionListener() {
			@Override
			public void valueChanged(TreeSelectionEvent e) {

			}
		});
	}

	/**
	 * õѺѺЭ
	 * 
	 * @return String
	 */
	public String getYaYunType() {
		if (yaYunButton.isSelected()) {
			return "Ѻ";
		} else {
			return "Э";
		}
	}

	/**
	 * õĬ
	 * 
	 * @return String
	 */
	public String getDefaultYunType() {
		return ConfigReader.getInstance().getDefaultYunType();
	}

	/**
	 * ָͨ͵õģ
	 * 
	 * @param str
	 *            String
	 * @return TreeModel
	 */
	public TreeModel getTreeModel(String str) {
		doc = DOM4jReader.getYunDOM(str);
		dtm = new DocumentTreeModel(doc);
		return dtm;
	}

	/**
	 * õ
	 * 
	 * @return String
	 */
	public String getYunType() {
		if (xinYunButton.isSelected()) {
			return "л";
		} else {
			return "";
		}
	}

	/**
	 * ʾ
	 */
	public void viewWords() {
		String word = getWordField.getText();
		if (word.equals("")) {
			return;
		}
		String yaYunType = getYaYunType();
		String yunType = getYunType();
		String result = null;
		try {
			if (yaYunType.equals("Ѻ")) {
				result = CoreMethods.getCommonableWordsFromWord(word, yunType);
			} else {
				result = CoreMethods.getOppositeWordsFromWord(word, yunType);
			}
		} catch (WordParserErrorException e) {
			ComponentManager.msgBox("δҵĿ֡");
		} catch (NotWordException e) {
			ComponentManager.msgBox("ҪĲЧ֣");
		}
		showWordsArea.setText(result);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == zhengYunButton) {
			yunTree.setModel(getTreeModel(""));
		} else if (e.getSource() == xinYunButton) {
			yunTree.setModel(getTreeModel("л"));
		} else if (e.getSource() == viewButton) {
			viewWords();
		} else if (e.getSource() == getWordField) {
			viewWords();
		}
	}

	@Override
	public boolean canClose() {
		return true;
	}

	@Override
	public String getPanelName() {
		return "";
	}
}
