package org.hiroyukki.bxp.yun;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

/**
 * ˷˲ҿͨϵֵķЭϵķֺͨʹϱõֵϢķ
 * ʱݴжлһڴҡҹ涨жһǷֵ֣Ļδ
 * 
 * @author hiroyukki
 * @since 1.0
 */
public class CoreMethods {
	/**
	 * һ֣ϸ׳쳣
	 * 
	 * @param word
	 *            String
	 * @param yunType
	 *            String
	 * @return wi WordInfo
	 */

	public static WordInfo getWordInfo(String word, String yunType) {
		if (word.length() != 1 && !word.endsWith("1")) {
			throw new NotWordException("ĿЧ");
		}
		WordInfo wi = null;

		if (yunType.equals("")) {
			wi = CiLinZhengYun.getWordsMap().get(word);
		} else if (yunType.equals("л")) {
			wi = ZhongHuaXinYun.getWordsMap().get(word);
		}
		if (wi == null) {
			throw new WordParserErrorException("ֽʧܣ" + word);
		}
		return wi;
	}

	/**
	 * õֵϲ
	 * 
	 * @param word
	 *            String
	 * @param yunType
	 *            String
	 * @return yunBu String
	 */
	public static String getYunBu(String word, String yunType) {
		WordInfo wi = getWordInfo(word, yunType);
		return wi.getYunBu();
	}

	/**
	 * õһֵƽƣһƽ֣Ϊ
	 * 
	 * @param word
	 *            String
	 * @param yunType
	 *            String
	 * @return String
	 */
	public static String getPingZe(String word, String yunType) {
		WordInfo wi = getWordInfo(word, yunType);
		return wi.getPingZe();
	}

	/**
	 * õֵϸ
	 * 
	 * @param word
	 *            String
	 * @param yunType
	 *            String
	 * @return String
	 */
	public static String getYunDetails(String word, String yunType) {
		WordInfo wi = getWordInfo(word, yunType);
		return wi.getYunDetails();
	}

	/**
	 * ֵͨõͬͬƽƵ֣ʱͬͬƽƵΪͬ
	 * 
	 * @param word
	 *            String
	 * @return map HashMap<String, YunDetails>
	 */
	public static String getCommonableWordsFromWord(String word, String yunType) {
		StringBuilder sb = new StringBuilder();
		HashMap<String, YunDetails> map;
		String yunBu = getYunBu(word, yunType);
		String pingZe = getPingZe(word, yunType);
		try {
			if (yunType.equals("")) {
				map = CiLinZhengYun.getInstance().get(yunBu);
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (yd.getType().equals(pingZe)) {
						sb.append(yd.getContent());
					}
				}
			} else if (yunType.equals("л")) {
				map = ZhongHuaXinYun.getInstance().get(yunBu);
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (yd.getType().equals(pingZe)) {
						sb.append(yd.getContent());
					}
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return sb.toString();
	}

	/**
	 * õͬƽƵ֣ʱͬƽƵΪЭ
	 * 
	 * @param word
	 *            String
	 * @param yunType
	 *            String
	 * @return map HashMap<String, YunDetails>
	 */
	public static String getOppositeWordsFromWord(String word, String yunType) {
		StringBuilder sb = new StringBuilder();
		HashMap<String, YunDetails> map;
		String yunBu = getYunBu(word, yunType);
		String pingZe = getPingZe(word, yunType);
		try {
			if (yunType.equals("")) {
				map = CiLinZhengYun.getInstance().get(yunBu);
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (!yd.getType().equals(pingZe)) {
						sb.append(yd.getContent());
					}
				}
			} else if (yunType.equals("л")) {
				map = ZhongHuaXinYun.getInstance().get(yunBu);
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (yd.getType().equals(pingZe)) {
						sb.append(yd.getContent());
					}
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return sb.toString();
	}

	/**
	 * 鿴һǷ֡
	 * 
	 * @param word
	 *            string
	 * @param yunType
	 *            String
	 * @return boolean
	 */
	public static boolean isControversialWord(String word, String yunType) {
		if (yunType.equals("")) {
			return CiLinZhengYun.getWordsMap().containsKey(word + "1");
		} else if (yunType.equals("л")) {
			return ZhongHuaXinYun.getWordsMap().containsKey(word + "1");
		}
		return false;
	}

	/**
	 * 鿴һǷƽ֣һֲҵеƽƲͬʱΪƽ
	 * 
	 * @param word
	 *            String
	 * @param yunType
	 *            String
	 * @return boolean
	 */
	public static boolean isPingZeControversialWord(String word, String yunType) {
		// һֲ֣Ȼƽ
		if (isControversialWord(word, yunType) == false) {
			return false;
		}
		// ֵֿƽƽ
		String sourceWord = word;
		String anotherWord = word + "1";
		WordInfo wi1 = getWordInfo(sourceWord, yunType);
		WordInfo wi2 = getWordInfo(anotherWord, yunType);
		if (!wi1.getPingZe().equals(wi2.getPingZe())) {
			return true;
		}
		return false;
	}

	/**
	 * ͨϲƽƵõͬͬƽƵ飬һõ˷
	 * 
	 * @param yunBu
	 *            String
	 * @param pingZe
	 *            String
	 * @param yunType
	 *            String
	 * @return String
	 */
	public static String getCommonableWordsFromYunBu(String yunBu,
			String pingZe, String yunType) {
		StringBuilder sb = new StringBuilder();
		HashMap<String, YunDetails> map;
		try {
			if (yunType.equals("")) {
				map = CiLinZhengYun.getInstance().get(yunBu);
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (yd.getType().equals(pingZe)) {
						sb.append(yd.getContent());
					}
				}
			} else if (yunType.equals("л")) {
				map = ZhongHuaXinYun.getInstance().get(yunBu);
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (yd.getType().equals(pingZe)) {
						sb.append(yd.getContent());
					}
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return sb.toString();
	}

	/**
	 * ͨϲƽƵõͬƽƵ飬һõ˷
	 * 
	 * @param yunBu
	 *            String
	 * @param pingZe
	 *            String
	 * @param yunType
	 *            String
	 * @return String
	 */
	public static String getOppositeWordsFromYunBu(String yunBu, String pingZe,
			String yunType) {
		StringBuilder sb = new StringBuilder();
		HashMap<String, YunDetails> map;
		try {
			if (yunType.equals("")) {
				map = CiLinZhengYun.getInstance().get(yunBu);
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (!yd.getType().equals(pingZe)) {
						sb.append(yd.getContent());
					}
				}
			} else if (yunType.equals("л")) {
				map = ZhongHuaXinYun.getInstance().get(yunBu);
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (!yd.getType().equals(pingZe)) {
						sb.append(yd.getContent());
					}
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return sb.toString();
	}

	/**
	 * ָͨģָͣϻϣõõͬͬƽϸб
	 * 
	 * @param ym
	 *            YunModel
	 * @param yunType
	 *            String
	 * @return ArrayList<YunDetails>
	 */
	public static ArrayList<YunDetails> getCommonableYunDetailssFromYunModel(
			YunModel ym, String yunType) {
		ArrayList<YunDetails> list = new ArrayList<YunDetails>();
		HashMap<String, YunDetails> map;
		try {
			if (yunType.equals("")) {
				map = CiLinZhengYun.getInstance().get(ym.getYunBu());
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (yd.getType().equals(ym.getPingZe())) {
						list.add(yd);
					}
				}
			} else if (yunType.equals("л")) {
				map = ZhongHuaXinYun.getInstance().get(ym.getYunBu());
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (yd.getType().equals(ym.getPingZe())) {
						list.add(yd);
					}
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return list;
	}

	/**
	 * ָͨģõпõͬƽƵϸ
	 * 
	 * @param ym
	 *            YunModel
	 * @param yunType
	 *            String
	 * @return ArrayList<YunDetails>
	 */
	public static ArrayList<YunDetails> getOppositeYunDetailssFromYunModel(
			YunModel ym, String yunType) {
		ArrayList<YunDetails> list = new ArrayList<YunDetails>();
		HashMap<String, YunDetails> map;
		try {
			if (yunType.equals("")) {
				map = CiLinZhengYun.getInstance().get(ym.getYunBu());
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (!yd.getType().equals(ym.getPingZe())) {
						list.add(yd);
					}
				}
			} else if (yunType.equals("л")) {
				map = ZhongHuaXinYun.getInstance().get(ym.getYunBu());
				Collection<YunDetails> c = map.values();
				for (YunDetails yd : c) {
					if (!yd.getType().equals(ym.getPingZe())) {
						list.add(yd);
					}
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return list;
	}

	/**
	 * ӴģõõͬͬƽƵַ
	 * 
	 * @param ym
	 *            YunModel
	 * @param yunType
	 *            String
	 * @return String
	 */
	public static String getCommonableWordsFromYunModel(YunModel ym,
			String yunType) {
		return getCommonableWordsFromYunBu(ym.getYunBu(), ym.getPingZe(),
				yunType);
	}

	/**
	 * ӴģõõͬƽƵַ
	 * 
	 * @param ym
	 *            YunModel
	 * @param yunType
	 *            String
	 * @return String
	 */
	public static String getOppositeWordsFromYunModel(YunModel ym,
			String yunType) {
		return getOppositeWordsFromYunBu(ym.getYunBu(), ym.getPingZe(), yunType);
	}

	/**
	 * һϢʵõһģʵ
	 * 
	 * @param wi
	 *            WordInfo
	 * @return YunModel
	 */
	public static YunModel getYunModelFromWordInfo(WordInfo wi) {
		YunModel ym = new YunModel();
		ym.setYunBu(wi.getYunBu());
		ym.setPingZe(wi.getPingZe());
		return ym;
	}
}
