package org.hiroyukki.tools.ui;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;

/**
 * Button extend from JButton, provide mouse listener, when mouse hover, show
 * some message.
 * 
 * @author hiroyukki
 * @version 2010.01.06
 */
public class HButton extends JButton {

	private static final long serialVersionUID = 1L;

	public HButton() {
		super();
		init();
	}

	/**
	 * Create button with default image icon
	 * 
	 * @param ico
	 *            Icon
	 */
	public HButton(Icon ico) {
		super(ico);
		init();
	}

	/**
	 * Create button with default text on it
	 * 
	 * @param str
	 *            String
	 */
	public HButton(String str) {
		super(str);
		init();
	}

	/**
	 * create button with default action.
	 * 
	 * @param a
	 *            javax.swing.Action
	 */
	public HButton(Action a) {
		super(a);
		init();
	}

	/**
	 * create button with default text and image icon
	 * 
	 * @param txt
	 *            String
	 * @param icon
	 *            Icon
	 */
	public HButton(String txt, Icon icon) {
		super(txt, icon);
		init();
	}

	/**
	 * Create button with default image icon and message, this message will show
	 * out when mouse hover
	 * 
	 * @param icon
	 *            Icon
	 * @param msg
	 *            String
	 */
	public HButton(Icon icon, String msg) {
		super(icon);
		this.message = msg;
		init();
	}

	private String message;

	private void init() {
		this.setToolTipText(message);
	}
}
