package org.hiroyukki.tools.ui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.border.TitledBorder;

import org.hiroyukki.tools.ui.img.*;

/**
 * Dialog for text area. provides search, replace operations.<br>
 * This class is SINGLETON.<br>
 * so before use it, need set which text area it will be servive.
 * 
 * @author hiroyukki
 * @version 2010.01.08
 */
public class HFindDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;

	/**
	 * mark find derection is forword
	 */
	public static final int FORWORD = 1;

	/**
	 * mark find derection is backword
	 */
	public static final int BACKWORD = 2;

	public static HFindDialog instance;
	private static final Integer I = 1;

	public static HFindDialog getInstance() {
		if (instance == null) {
			synchronized (I) {
				if (instance == null) {
					instance = new HFindDialog();
				}
			}
		}
		return instance;
	}

	/**
	 * define search direction, forword or backword.
	 */
	private int findType;

	private HTextField findField = new HTextField(10);
	private HTextField replaceField = new HTextField(10);
	private JPanel findPanel = new JPanel();
	private JPanel replacePanel = new JPanel();
	private JPanel fieldPanel = new JPanel();

	private JPanel radioPanel = new JPanel();
	private JRadioButton forwordBtn = new JRadioButton("ǰ");
	private JRadioButton backwordBtn = new JRadioButton("");

	private JPanel btnPanel = new JPanel();
	private JButton findBtn = new JButton("");
	private JButton replaceBtn = new JButton("滻");
	private JButton replaceAllBtn = new JButton("ȫ滻");
	private JButton closeBtn = new JButton("ر");

	private JTextArea jta;

	private HFindDialog() {
		findType = FORWORD;
		findField.addActionListener(this);
		findPanel.setLayout(new BorderLayout());
		findPanel.add(new JLabel("    ң"), BorderLayout.WEST);
		findPanel.add(findField, BorderLayout.CENTER);
		replacePanel.setLayout(new BorderLayout());
		replacePanel.add(new JLabel("滻Ϊ"), BorderLayout.WEST);
		replacePanel.add(replaceField, BorderLayout.CENTER);
		fieldPanel.setLayout(new GridLayout(2, 1, 5, 5));
		fieldPanel.add(findPanel);
		fieldPanel.add(replacePanel);
		fieldPanel.setBorder(new TitledBorder(""));

		ButtonGroup bg = new ButtonGroup();
		bg.add(backwordBtn);
		bg.add(forwordBtn);
		forwordBtn.setSelected(true);
		radioPanel.setBorder(new TitledBorder("ҷ"));
		radioPanel.add(forwordBtn);
		radioPanel.add(backwordBtn);

		findField.addActionListener(this);
		findBtn.addActionListener(this);
		replaceBtn.addActionListener(this);
		replaceAllBtn.addActionListener(this);
		closeBtn.addActionListener(this);
		forwordBtn.addActionListener(this);
		backwordBtn.addActionListener(this);

		btnPanel.setLayout(new GridLayout(2, 2, 5, 5));
		btnPanel.setBorder(new TitledBorder(""));
		btnPanel.add(findBtn);
		btnPanel.add(replaceBtn);
		btnPanel.add(replaceAllBtn);
		btnPanel.add(closeBtn);

		setAlwaysOnTop(true);
		setTitle("/滻");
		setSize(200, 320);
		setResizable(false);
		setIconImage(ImageLoader.getImage("logo.png"));
		add(fieldPanel, BorderLayout.NORTH);
		add(radioPanel, BorderLayout.CENTER);
		add(btnPanel, BorderLayout.SOUTH);
		pack();
		ComponentManager.moveToCenter(this);
	}

	/**
	 * Find target string from content of JTextArea
	 */
	private void find() {
		String source = findField.getText();
		if ("".equals(source)) {
			ComponentManager.msgBox("Ҫҵݣ");
			return;
		}
		String content = jta.getText();
		int length = source.length();
		int position = jta.getCaretPosition();
		// ǰ
		if (findType == FORWORD) {
			String buffer = content.substring(position, content.length());
			int bufferPosition = buffer.indexOf(source);
			if (bufferPosition == -1) {
				ComponentManager.msgBox("δҵַ\"" + source + "\"!");
				return;
			}
			jta.select(position + bufferPosition, position + bufferPosition
					+ length);
		} else if (findType == BACKWORD) {
			String buffer = content.substring(0, position);
			if (buffer.indexOf(source) == -1) {
				ComponentManager.msgBox("δҵַ\"" + source + "\"!");
				return;
			}
			StringBuilder sbBuffer = new StringBuilder(buffer);
			StringBuilder sbSource = new StringBuilder(source);
			sbBuffer.reverse();
			sbSource.reverse();
			int bufferPosition = sbBuffer.indexOf(sbSource.toString());
			bufferPosition = buffer.length() - bufferPosition - length;
			jta.setCaretPosition(bufferPosition + length);
			jta.moveCaretPosition(bufferPosition);
		}
	}

	/**
	 * Replace selected string to appoint string
	 */
	private void replace() {
		String target = replaceField.getText();
		if (target.equals("")) {
			ComponentManager.msgBox("Ҫ滻ɵݣ");
			return;
		}
		if (jta.getSelectedText() == null) {
			return;
		}
		int start = jta.getSelectionStart();
		int end = jta.getSelectionEnd();
		int length = target.length();
		StringBuilder sb = new StringBuilder(jta.getText());
		sb.replace(start, end, target);
		jta.setText(sb.toString());
		jta.select(start, start + length);
	}

	/**
	 * Replace all same string to appoint string
	 */
	private void replaceAll() {
		String source = findField.getText();
		String target = replaceField.getText();
		if (source.equals("") || target.equals("")) {
			ComponentManager.msgBox("ԴַĿַΪգ");
			return;
		}
		String content = jta.getText();
		if (content.indexOf(source) == -1) {
			ComponentManager.msgBox("δҵַ\"" + source + "\"");
			return;
		}
		content = content.replaceAll(source, target);
		jta.setText(content);
	}

	public JTextArea getJta() {
		return jta;
	}

	public void setJta(JTextArea jta) {
		this.jta = jta;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == findField) {
		} else if (e.getSource() == forwordBtn) {
			findType = FORWORD;
		} else if (e.getSource() == backwordBtn) {
			findType = BACKWORD;
		} else if (e.getSource() == findBtn || e.getSource() == findField) {
			find();
		} else if (e.getSource() == replaceBtn) {
			if (jta.isEditable()) {
				replace();
			}
		} else if (e.getSource() == replaceAllBtn) {
			if (jta.isEditable()) {
				replaceAll();
			}
		} else if (e.getSource() == closeBtn) {
			setVisible(false);
		}
	}
}
